<?php
/**
 * Copyright ETS Software Technology Co., Ltd
 *
 * NOTICE OF LICENSE
 *
 * This file is not open source! Each license that you purchased is only available for 1 website only.
 * If you want to use this file on more websites (or projects), you need to purchase additional licenses.
 * You are not allowed to redistribute, resell, lease, license, sub-license or offer our resources to any third party.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future.
 *
 * @author ETS Software Technology Co., Ltd
 * @copyright  ETS Software Technology Co., Ltd
 * @license    Valid for 1 website (or project) for each purchase of license
 */
if (!defined('_PS_VERSION_')) {
    exit;
}
class EtsAwuDefine
{
    /**
     * @var array
     */
    protected static $images_types_name_cache = [];
    /**
     * @var \Context|\ContextCore
     */
    public $context;
    /**
     * @var \Ets_awesomeurl
     */
    private $module;
    /**
     * @var bool
     */
    public $is17;
    /**
     * @var self
     */
    private static $instance;
    /**
     * @var string[]
     */
    private $hooks = [
        'displayBackOfficeHeader',
        'actionObjectUpdateBefore',
        'actionDispatcherBefore',
        'displayHeader',
        'actionObjectUpdateAfter',
        'actionAdminMetaControllerUpdate_optionsBefore',
        'actionAdminMetaControllerUpdate_optionsAfter',
        'actionAdminEtsAwuUrlRedirectsFormModifier',
        'actionObjectAddAfter',
        'actionProductSave',
        'displayAdminProductsSeoStepBottom',
        'actionMetaFormBuilderModifier',
        'actionBeforeCreateMetaFormHandler',
        'actionBeforeUpdateMetaFormHandler',
        'actionCmsPageCategoryFormBuilderModifier',
        'actionBeforeCreateCmsPageCategoryFormHandler',
        'actionBeforeUpdateCmsPageCategoryFormHandler',
        'actionCmsPageFormBuilderModifier',
        'actionBeforeCreateCmsPageFormHandler',
        'actionBeforeUpdateCmsPageFormHandler',
        'actionRootCategoryFormBuilderModifier',
        'actionBeforeCreateRootCategoryFormHandler',
        'actionBeforeUpdateRootCategoryFormHandler',
        'actionCategoryFormBuilderModifier',
        'actionBeforeCreateCategoryFormHandler',
        'actionBeforeUpdateCategoryFormHandler',
        'actionObjectAddBefore',
        'actionProductFormBuilderModifier',
    ];
    /**
     * List admin controller will be added override template & custom js.
     *
     * @var string[]
     */
    private $adminControllerOverrides = [
        'AdminProduct',
        'AdminManufacturer',
        'AdminCmsContent',
        'AdminCategories',
        'AdminSuppliers',
        'AdminMeta',
    ];

    public function __construct()
    {
        $context = Context::getContext();
        $this->context = $context;
        $this->is17 = version_compare('1.7.0.0', _PS_VERSION_, '<=');
    }

    public function l($string)
    {
        return Translate::getModuleTranslation('ets_awesomeurl', $string, pathinfo(__FILE__, PATHINFO_FILENAME));
    }

    public function display($template)
    {
        return $this->getModule()->display($this->getModule()->getLocalPath(), $template);
    }

    /**
     * @return self
     */
    public static function getInstance()
    {
        if (!self::$instance instanceof self) {
            self::$instance = new self();
        }

        return self::$instance;
    }

    /**
     * @return string[]
     */
    public function getListAdminControllerOverrides()
    {
        return $this->adminControllerOverrides;
    }

    public function adminControllers()
    {
        return [
            'AdminEtsAwuUrlRedirects' => [
                'title' => $this->l('URL Redirection'),
                'title_text' => 'URL Redirection',
                'class' => '',
                'icon' => 'ets-sidebar-icon-url-redirect',
            ],
            'AdminEtsAwuDuplicateUrls' => [
                'title' => $this->l('Check Duplicate URLs'),
                'title_text' => 'Check Duplicate URLs',
                'class' => '',
                'icon' => 'ets-sidebar-icon-duplicate-url',
            ],
            'AdminEtsAwuSearchAppearanceSitemap' => [
                'title' => $this->l('Sitemap'),
                'title_text' => 'Sitemap',
                'class' => '',
                'icon' => 'ets-sitemap',
            ],
            'AdminEtsAwuSearchAppearanceRSS' => [
                'title' => $this->l('Rss'),
                'title_text' => 'Rss',
                'class' => '',
                'icon' => 'ets-rss',
            ],
        ];
    }

    public function installDb()
    {
        $tbl_url_redirect = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . "ets_awu_redirect` (
            `id_ets_awu_redirect` int(11) unsigned NOT NULL AUTO_INCREMENT,
            `url` VARCHAR(255) NOT NULL,
            `target` VARCHAR(255) NOT NULL,
            `type` ENUM('301', '302', '303', '404') DEFAULT NULL,
            `active` INT(1) DEFAULT 1,
            `id_shop` INT(11) NOT NULL,
            PRIMARY KEY (`id_ets_awu_redirect`),
            INDEX (`active`,`id_shop`)
        ) ENGINE=" . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci';
        $tbl_seo_product = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'ets_awu_product` (
            `id_ets_awu_product` int(11) unsigned NOT NULL AUTO_INCREMENT,
            `id_product` INT(10) UNSIGNED NOT NULL,
            `id_shop` INT(11) NOT NULL,
            `id_lang` INT(11) NOT NULL,
            `key_phrase` VARCHAR(191) DEFAULT NULL,
            `minor_key_phrase` VARCHAR(191) DEFAULT NULL,
            `allow_search` INT(1) UNSIGNED DEFAULT 2,
            `allow_flw_link` INT(1) UNSIGNED DEFAULT 1,
            `meta_robots_adv` VARCHAR(191) DEFAULT NULL,
            `meta_keywords` VARCHAR(191) DEFAULT NULL,
            `canonical_url` VARCHAR(191) DEFAULT NULL,
            `seo_score` INT(3) UNSIGNED  DEFAULT NULL,
            `readability_score` INT(3) UNSIGNED DEFAULT NULL,
            `score_analysis` TEXT DEFAULT NULL,
            `content_analysis` TEXT DEFAULT NULL,
            `social_title` VARCHAR(191) DEFAULT NULL,
            `social_desc` TEXT DEFAULT NULL,
            `social_img` VARCHAR(191) DEFAULT NULL,
            PRIMARY KEY (`id_ets_awu_product`),
            UNIQUE KEY `ets_awu_psl` (id_product, id_shop, id_lang)
        ) ENGINE=' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci';

        $tbl_seo_category = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'ets_awu_category` (
            `id_ets_awu_category` int(11) unsigned NOT NULL AUTO_INCREMENT,
            `id_category` INT(10) UNSIGNED NOT NULL,
            `id_shop` INT(11) NOT NULL,
            `id_lang` INT(11) NOT NULL,
            `key_phrase` VARCHAR(191) DEFAULT NULL,
            `minor_key_phrase` VARCHAR(191) DEFAULT NULL,
            `allow_search` INT(1) UNSIGNED DEFAULT 2,
            `allow_flw_link` INT(1) UNSIGNED DEFAULT 1,
            `meta_robots_adv` VARCHAR(191) DEFAULT NULL,
            `meta_keywords` VARCHAR(191) DEFAULT NULL,
            `canonical_url` VARCHAR(191) DEFAULT NULL,
            `seo_score` INT(3) UNSIGNED  DEFAULT NULL,
            `readability_score` INT(3) UNSIGNED DEFAULT NULL,
            `score_analysis` TEXT DEFAULT NULL,
            `content_analysis` TEXT DEFAULT NULL,
            `social_title` VARCHAR(191) DEFAULT NULL,
            `social_desc` TEXT DEFAULT NULL,
            `social_img` VARCHAR(191) DEFAULT NULL,
            PRIMARY KEY (`id_ets_awu_category`),
            UNIQUE KEY `ets_awu_csl` (id_category, id_shop, id_lang)
        ) ENGINE=' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci';

        $tbl_seo_cms = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'ets_awu_cms` (
            `id_ets_awu_cms` int(11) unsigned NOT NULL AUTO_INCREMENT,
            `id_cms` INT(10) UNSIGNED NOT NULL,
            `id_shop` INT(11) NOT NULL,
            `id_lang` INT(11) NOT NULL,
            `key_phrase` VARCHAR(191) DEFAULT NULL,
            `minor_key_phrase` VARCHAR(191) DEFAULT NULL,
            `allow_search` INT(1) UNSIGNED DEFAULT 2,
            `allow_flw_link` INT(1) UNSIGNED DEFAULT 1,
            `meta_robots_adv` VARCHAR(191) DEFAULT NULL,
            `meta_keywords` VARCHAR(191) DEFAULT NULL,
            `canonical_url` VARCHAR(191) DEFAULT NULL,
            `seo_score` INT(3) UNSIGNED DEFAULT NULL,
            `readability_score` INT(3) UNSIGNED DEFAULT NULL,
            `score_analysis` TEXT DEFAULT NULL,
            `content_analysis` TEXT DEFAULT NULL,
            `social_title` VARCHAR(191) DEFAULT NULL,
            `social_desc` TEXT DEFAULT NULL,
            `social_img` VARCHAR(191) DEFAULT NULL,
            PRIMARY KEY (`id_ets_awu_cms`),
            UNIQUE KEY `ets_awu_csl` (id_cms, id_shop, id_lang)
        ) ENGINE=' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci';

        $tbl_seo_cms_category = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'ets_awu_cms_category` (
            `id_ets_awu_cms_category` int(11) unsigned NOT NULL AUTO_INCREMENT,
            `id_cms_category` INT(10) UNSIGNED NOT NULL,
            `id_shop` INT(11) NOT NULL,
            `id_lang` INT(11) NOT NULL,
            `key_phrase` VARCHAR(191) DEFAULT NULL,
            `minor_key_phrase` VARCHAR(191) DEFAULT NULL,
            `allow_search` INT(1) UNSIGNED DEFAULT 2,
            `allow_flw_link` INT(1) UNSIGNED DEFAULT 1,
            `meta_robots_adv` VARCHAR(191) DEFAULT NULL,
            `meta_keywords` VARCHAR(191) DEFAULT NULL,
            `canonical_url` VARCHAR(191) DEFAULT NULL,
            `seo_score` INT(3) UNSIGNED DEFAULT NULL,
            `readability_score` INT(3) UNSIGNED DEFAULT NULL,
            `score_analysis` TEXT DEFAULT NULL,
            `content_analysis` TEXT DEFAULT NULL,
            `social_title` VARCHAR(191) DEFAULT NULL,
            `social_desc` TEXT DEFAULT NULL,
            `social_img` VARCHAR(191) DEFAULT NULL,
            PRIMARY KEY (`id_ets_awu_cms_category`),
            UNIQUE KEY `ets_awu_ccsl` (id_cms_category, id_shop, id_lang)
        ) ENGINE=' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci';

        $tbl_seo_meta = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'ets_awu_meta` (
            `id_ets_awu_meta` int(11) unsigned NOT NULL AUTO_INCREMENT,
            `id_meta` INT(10) UNSIGNED NOT NULL,
            `id_shop` INT(11) NOT NULL,
            `id_lang` INT(11) NOT NULL,
            `key_phrase` VARCHAR(191) DEFAULT NULL,
            `minor_key_phrase` VARCHAR(191) DEFAULT NULL,
            `allow_search` INT(1) UNSIGNED DEFAULT 2,
            `allow_flw_link` INT(1) UNSIGNED DEFAULT 1,
            `meta_robots_adv` VARCHAR(191) DEFAULT NULL,
            `meta_keywords` VARCHAR(191) DEFAULT NULL,
            `canonical_url` VARCHAR(191) DEFAULT NULL,
            `seo_score` INT(3) UNSIGNED DEFAULT NULL,
            `readability_score` INT(3) UNSIGNED DEFAULT NULL,
            `score_analysis` TEXT DEFAULT NULL,
            `content_analysis` TEXT DEFAULT NULL,
            `social_title` VARCHAR(191) DEFAULT NULL,
            `social_desc` TEXT DEFAULT NULL,
            `social_img` VARCHAR(191) DEFAULT NULL,
            PRIMARY KEY (`id_ets_awu_meta`),
            UNIQUE KEY `ets_awu_msl` (id_meta, id_shop, id_lang)
        ) ENGINE=' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci';

        $tbl_seo_supplier = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'ets_awu_supplier` (
            `id_ets_awu_supplier` int(11) unsigned NOT NULL AUTO_INCREMENT,
            `id_supplier` INT(10) UNSIGNED NOT NULL,
            `id_shop` INT(11) NOT NULL,
            `id_lang` INT(11) NOT NULL,
            `key_phrase` VARCHAR(191) DEFAULT NULL,
            `minor_key_phrase` VARCHAR(191) DEFAULT NULL,
            `allow_search` INT(1) UNSIGNED DEFAULT 2,
            `allow_flw_link` INT(1) UNSIGNED DEFAULT 1,
            `meta_robots_adv` VARCHAR(191) DEFAULT NULL,
            `meta_keywords` VARCHAR(191) DEFAULT NULL,
            `canonical_url` VARCHAR(191) DEFAULT NULL,
            `seo_score` INT(3) UNSIGNED DEFAULT NULL,
            `readability_score` INT(3) UNSIGNED DEFAULT NULL,
            `score_analysis` TEXT DEFAULT NULL,
            `content_analysis` TEXT DEFAULT NULL,
            `social_title` VARCHAR(191) DEFAULT NULL,
            `social_desc` TEXT DEFAULT NULL,
            `social_img` VARCHAR(191) DEFAULT NULL,
            PRIMARY KEY (`id_ets_awu_supplier`),
            UNIQUE KEY `ets_awu_ssl` (id_supplier, id_shop, id_lang)
        ) ENGINE=' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci';

        $tbl_seo_manufacturer = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'ets_awu_manufacturer` (
            `id_ets_awu_manufacturer` int(11) unsigned NOT NULL AUTO_INCREMENT,
            `id_manufacturer` INT(10) UNSIGNED NOT NULL,
            `id_shop` INT(11) NOT NULL,
            `id_lang` INT(11) NOT NULL,
            `key_phrase` VARCHAR(191) DEFAULT NULL,
            `minor_key_phrase` VARCHAR(191) DEFAULT NULL,
            `allow_search` INT(1) UNSIGNED DEFAULT 2,
            `allow_flw_link` INT(1) UNSIGNED DEFAULT 1,
            `meta_robots_adv` VARCHAR(191) DEFAULT NULL,
            `meta_keywords` VARCHAR(191) DEFAULT NULL,
            `canonical_url` VARCHAR(191) DEFAULT NULL,
            `seo_score` INT(3) UNSIGNED DEFAULT NULL,
            `readability_score` INT(3) UNSIGNED DEFAULT NULL,
            `score_analysis` TEXT DEFAULT NULL,
            `content_analysis` TEXT DEFAULT NULL,
            `social_title` VARCHAR(191) DEFAULT NULL,
            `social_desc` TEXT DEFAULT NULL,
            `social_img` VARCHAR(191) DEFAULT NULL,
            PRIMARY KEY (`id_ets_awu_manufacturer`),
            UNIQUE KEY `ets_awu_msl` (id_manufacturer, id_shop, id_lang)
        ) ENGINE=' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci';

        return Db::getInstance()->execute($tbl_url_redirect)
        && Db::getInstance()->execute($tbl_seo_product)
        && Db::getInstance()->execute($tbl_seo_category)
        && Db::getInstance()->execute($tbl_seo_manufacturer)
        && Db::getInstance()->execute($tbl_seo_supplier)
        && Db::getInstance()->execute($tbl_seo_cms)
        && Db::getInstance()->execute($tbl_seo_cms_category)
        && Db::getInstance()->execute($tbl_seo_meta);
    }

    public function uninstallDb()
    {
        $tables = [
            'ets_awu_redirect',
            'ets_awu_product',
            'ets_awu_category',
            'ets_awu_cms',
            'ets_awu_cms_category',
            'ets_awu_meta',
            'ets_awu_supplier',
            'ets_awu_manufacturer',
        ];
        foreach ($tables as $table) {
            Db::getInstance()->execute('DROP TABLE IF EXISTS `' . _DB_PREFIX_ . bqSQL($table) . '`');
        }

        return true;
    }

    public function getFieldConfig()
    {
        return [
            'url_redirect_setting' => [
                'ETS_AWU_ENABLE_URL_REDIRECT' => [
                    'title' => $this->l('Enabled'),
                    'validation' => 'isBool',
                    'cast' => 'intval',
                    'type' => 'bool',
                    'default' => 0,
                    'no_multishop_checkbox' => true,
                    'desc' => $this->l('Enable or disable URL redirection for redirect rules defined below'),
                ],
            ],
            'ps_extra' => [
                'ETS_AWU_ENABLE_REMOVE_ID_IN_URL' => [
                    'title' => $this->l('Remove ID in URL'),
                    'validation' => 'isBool',
                    'cast' => 'intval',
                    'type' => 'bool',
                    'default' => 0,
                ],
                'ETS_AWU_ENABLE_REMOVE_LANG_CODE_IN_URL' => [
                    'title' => $this->l('Remove ISO code in URL for default language'),
                    'validation' => 'isBool',
                    'cast' => 'intval',
                    'type' => 'bool',
                    'default' => 0,
                ],
                'ETS_AWU_ENABLE_REMOVE_ATTR_ALIAS' => [
                    'title' => $this->l('Remove attribute alias in URL'),
                    'validation' => 'isBool',
                    'cast' => 'intval',
                    'type' => 'bool',
                    'default' => 0,
                ],
                'ETS_AWU_ENABLE_REMOVE_ID_ATTR_ALIAS' => [
                    'title' => $this->l('Remove ID attribute alias in URL'),
                    'validation' => 'isBool',
                    'cast' => 'intval',
                    'type' => 'bool',
                    'default' => 0,
                ],
                'ETS_AWU_ENABLE_REDIRECT_NOTFOUND' => [
                    'title' => $this->l('Redirect all old URLs to new URLs (keep your page rankings and backlinks)'),
                    'validation' => 'isBool',
                    'cast' => 'intval',
                    'type' => 'bool',
                    'default' => 1,
                ],
                'ETS_AWU_REDIRECT_STATUS_CODE' => [
                    'title' => $this->l('Redirect type'),
                    'validation' => 'isInt',
                    'type' => 'select',
                    'identifier' => 'value',
                    'list' => [
                        [
                            'name' => $this->l('302 Moved Temporarily (recommended while setting up your store)'),
                            'value' => '302',
                        ],
                        [
                            'name' => $this->l('301 Moved Permanently (recommended once you have gone live)'),
                            'value' => '301',
                        ],
                    ],
                    'default' => '302',
                ],
            ],
            'sitemap_setting' => [
                'ETS_AWU_ENABLE_XML_SITEMAP' => [
                    'title' => $this->l('Enable sitemaps'),
                    'validation' => 'isBool',
                    'cast' => 'intval',
                    'type' => 'bool',
                    'default' => 1,
                    'no_multishop_checkbox' => true,
                ],
                'ETS_AWU_SITEMAP_PRIMARY' => [
                    'title' => $this->l('Primary sitemap'),
                    'type' => 'text',
                    'no_multishop_checkbox' => true,
                ],
                'ETS_AWU_SITEMAP_LANG' => [
                    'title' => $this->l('Sitemap by languages'),
                    'type' => 'text',
                    'no_multishop_checkbox' => true,
                ],
                'ETS_AWU_SITEMAP_PRIORITY' => [
                    'title' => $this->l('Priority / Change frequency'),
                    'validation' => 'isUnsignedFloat',
                    'type' => 'text',
                    'default' => 0.5,
                    'no_multishop_checkbox' => true,
                ],
                'ETS_AWU_SITEMAP_OPTION' => [
                    'title' => $this->l('Pages to include in sitemap'),
                    'type' => 'text',
                    'required' => 1,
                    'default' => 'product,category,cms,cms_category,manufacturer,supplier,meta',
                    'no_multishop_checkbox' => true,
                ],
                'ETS_AWU_PROD_SITEMAP_LIMIT' => [
                    'title' => $this->l('Number product per page in sitemap pagination'),
                    'validation' => 'isUnsignedInt',
                    'type' => 'text',
                    'default' => 250,
                    'no_multishop_checkbox' => true,
                    'desc' => $this->l('Leave blank to include all products in one sitemap (not recommended for large catalog)'),
                ],
                'ETS_AWU_CRONJOB_TOKEN' => [
                    'title' => $this->l('Token to run cronjob'),
                    'validation' => 'isString',
                    'type' => 'text',
                    'default' => Tools::passwdGen(12),
                    'no_multishop_checkbox' => true,
                    'desc' => $this->l('Leave blank to include all products in one sitemap (not recommended for large catalog)'),
                ],
            ],
            'sitemap_value' => [
                'ETS_AWU_SITEMAP_PRIORITY_PRODUCT' => [
                    'title' => '',
                    'type' => 'text',
                    'default' => 0.9,
                    'no_multishop_checkbox' => true,
                ],
                'ETS_AWU_SITEMAP_PRIORITY_CATEGORY' => [
                    'title' => '',
                    'type' => 'text',
                    'default' => 0.8,
                    'no_multishop_checkbox' => true,
                ],
                'ETS_AWU_SITEMAP_PRIORITY_CMS' => [
                    'title' => '',
                    'type' => 'text',
                    'default' => 0.1,
                    'no_multishop_checkbox' => true,
                ],
                'ETS_AWU_SITEMAP_PRIORITY_CMS_CATEGORY' => [
                    'title' => '',
                    'type' => 'text',
                    'default' => 0.1,
                    'no_multishop_checkbox' => true,
                ],
                'ETS_AWU_SITEMAP_PRIORITY_META' => [
                    'title' => '',
                    'type' => 'text',
                    'default' => 0.1,
                    'no_multishop_checkbox' => true,
                ],
                'ETS_AWU_SITEMAP_PRIORITY_SUPPLIER' => [
                    'title' => '',
                    'type' => 'text',
                    'default' => 0.1,
                    'no_multishop_checkbox' => true,
                ],
                'ETS_AWU_SITEMAP_PRIORITY_MANUFACTURER' => [
                    'title' => '',
                    'type' => 'text',
                    'default' => 0.1,
                    'no_multishop_checkbox' => true,
                ],
            ],
            'sitemap_freq' => [
                'ETS_AWU_SITEMAP_FREQ_PRODUCT' => [
                    'title' => '',
                    'type' => 'text',
                    'default' => 'weekly',
                    'no_multishop_checkbox' => true,
                ],
                'ETS_AWU_SITEMAP_FREQ_CATEGORY' => [
                    'title' => '',
                    'type' => 'text',
                    'default' => 'weekly',
                    'no_multishop_checkbox' => true,
                ],
                'ETS_AWU_SITEMAP_FREQ_CMS' => [
                    'title' => '',
                    'type' => 'text',
                    'default' => 'weekly',
                    'no_multishop_checkbox' => true,
                ],
                'ETS_AWU_SITEMAP_FREQ_CMS_CATEGORY' => [
                    'title' => '',
                    'type' => 'text',
                    'default' => 'weekly',
                    'no_multishop_checkbox' => true,
                ],
                'ETS_AWU_SITEMAP_FREQ_META' => [
                    'title' => '',
                    'type' => 'text',
                    'default' => 'weekly',
                    'no_multishop_checkbox' => true,
                ],
                'ETS_AWU_SITEMAP_FREQ_SUPPLIER' => [
                    'title' => '',
                    'type' => 'text',
                    'default' => 'weekly',
                    'no_multishop_checkbox' => true,
                ],
                'ETS_AWU_SITEMAP_FREQ_MANUFACTURER' => [
                    'title' => '',
                    'type' => 'text',
                    'default' => 'weekly',
                    'no_multishop_checkbox' => true,
                ],
            ],
            'rss_setting' => [
                'ETS_AWU_RSS_ENABLE' => [
                    'title' => $this->l('Enable RSS feed'),
                    'validation' => 'isBool',
                    'cast' => 'intval',
                    'type' => 'bool',
                    'default' => 1,
                    'required' => true,
                    'no_multishop_checkbox' => true,
                ],
                'ETS_AWU_RSS_OPTION' => [
                    'title' => $this->l('Pages to include in RSS'),
                    'validation' => 'isString',
                    'type' => 'text',
                    'default' => 'product_category,cms_category,all_products,new_products,special_products,popular_products',
                    'no_multishop_checkbox' => true,
                ],
                'ETS_AWU_RSS_CONTENT_BEFORE' => [
                    'title' => $this->l('Content to put before each item in the feed'),
                    'validation' => 'isString',
                    'type' => 'textareaLang',
                    'rows' => 5,
                    'no_multishop_checkbox' => true,
                ],
                'ETS_AWU_RSS_CONTENT_AFTER' => [
                    'title' => $this->l('Content to put after each item in the feed'),
                    'validation' => 'isString',
                    'type' => 'textareaLang',
                    'rows' => 5,
                    'no_multishop_checkbox' => true,
                ],
                'ETS_AWU_RSS_LINK' => [
                    'title' => $this->l('RSS link(s)'),
                    'type' => 'text',
                    'no_multishop_checkbox' => true,
                ],
                'ETS_AWU_RSS_POST_LIMIT' => [
                    'title' => $this->l('Item limit (the number of latest added items to display)'),
                    'validation' => 'isString',
                    'type' => 'text',
                    'default' => '100',
                    'desc' => $this->l('Leave blank to display all items (not recommended for large catalog)'),
                    'no_multishop_checkbox' => true,
                ],
            ],
        ];
    }

    public function urlRules()
    {
        return [
            'category_rule' => [
                'rule' => $this->getConfigRule('category_rule', '{id}-{rewrite}'),
                'new_rule' => $this->getConfigRule('category_rule', '{rewrite}', true),
                'desc_rule' => $this->l('Keywords: id* , rewrite , meta_keywords , meta_title'),
                'desc_new_rule' => $this->l('Keywords: id , rewrite* , meta_keywords , parent_rewrite'),
            ],
            'supplier_rule' => [
                'rule' => $this->getConfigRule('supplier_rule', 'supplier/{id}-{rewrite}'),
                'new_rule' => $this->getConfigRule('supplier_rule', 'supplier/{rewrite}', true),
                'desc_rule' => $this->l('Keywords: id* , rewrite , meta_keywords , meta_title'),
                'desc_new_rule' => $this->l('Keywords: id , rewrite* , meta_keywords '),
            ],
            'manufacturer_rule' => [
                'rule' => $this->getConfigRule('manufacturer_rule', 'brand/{id}-{rewrite}'),
                'new_rule' => $this->getConfigRule('manufacturer_rule', 'brand/{rewrite}', true),
                'desc_rule' => $this->l('Keywords: id* , rewrite , meta_keywords , meta_title'),
                'desc_new_rule' => $this->l('Keywords: id , rewrite* , meta_keywords'),
            ],
            'cms_rule' => [
                'rule' => $this->getConfigRule('cms_rule', 'content/{id}-{rewrite}'),
                'new_rule' => $this->getConfigRule('cms_rule', 'content/{rewrite}', true),
                'desc_rule' => $this->l('Keywords: id* , rewrite , meta_keywords , meta_title'),
                'desc_new_rule' => $this->l('Keywords: id , rewrite* , meta_keywords'),
            ],
            'cms_category_rule' => [
                'rule' => $this->getConfigRule('cms_category_rule', 'content/category/{id}-{rewrite}'),
                'new_rule' => $this->getConfigRule('cms_category_rule', 'content/category/{rewrite}', true),
                'desc_rule' => $this->l('Keywords: id* , rewrite , meta_keywords , meta_title'),
                'desc_new_rule' => $this->l('Keywords: id , rewrite* , meta_keywords'),
            ],
            'module' => [
                'rule' => $this->getConfigRule('module', 'module/{module}{/:controller}'),
                'new_rule' => $this->getConfigRule('module', 'module/{module}{/:controller}', true),
                'desc_rule' => $this->l('Keywords: module* , controller*'),
                'desc_new_rule' => $this->l('Keywords: module* , controller*'),
            ],
            'product_rule' => [
                'rule' => $this->getConfigRule('product_rule', $this->is17 ? '{category:/}{id}{-:id_product_attribute}-{rewrite}{-:ean13}.html' : '{category:/}{id}-{rewrite}{-:ean13}.html'),
                'new_rule' => $this->getConfigRule('product_rule', '{category}/{rewrite}', true),
                'desc_rule' => $this->is17 ? $this->l('Keywords: id* , id_product_attribute* , rewrite* , ean13 , category , categories , reference , meta_keywords , meta_title , manufacturer , supplier , price , tags') : $this->l('Keywords: id* , rewrite* , ean13 , category , categories , reference , meta_keywords , meta_title , manufacturer , supplier , price , tags'),
                'desc_new_rule' => $this->is17 ? $this->l('Keywords: id , id_product_attribute , rewrite* , ean13 , category, categories , reference , meta_keywords , manufacturer , supplier , price , tags') : $this->l('Keywords: id  , rewrite* , ean13 , category, categories , reference , meta_keywords , manufacturer , supplier , price , tags'),
            ],
            /* Must be after the product and category rules in order to avoid conflict */
            'layered_rule' => [
                'rule' => $this->getConfigRule('layered_rule', '{id}-{rewrite}{/:selected_filters}'),
                'new_rule' => $this->getConfigRule('layered_rule', '{rewrite}/filter/{selected_filters}', true),
                'desc_rule' => $this->l('Keywords: id* , selected_filters* , rewrite , meta_keywords , meta_title'),
                'desc_new_rule' => $this->l('Keywords: id , selected_filters* , rewrite* , meta_keywords'),
            ],
        ];
    }

    public function getConfigRule($rule, $default = null, $no_id = false)
    {
        $config = $this->seo_url_schema_configs();
        if ($no_id) {
            return ($data = Configuration::get($config[$rule]['no_id'])) && (('module' !== $rule && !preg_match('/\{id\}/', $data)) || 'module' == $rule) ? Configuration::get($config[$rule]['no_id']) : $default;
        }

        if (($nearest = Configuration::get($config[$rule]['name'])) && (('module' !== $rule && preg_match('/\{id\}/', $nearest)) || 'module' == $rule)) {
            return $nearest;
        } elseif (($old = Configuration::get($config[$rule]['old_name'])) && (('module' !== $rule && preg_match('/\{id\}/', $old)) || 'module' == $rule)) {
            return $old;
        } elseif (($root = Configuration::get($config[$rule]['root_name'])) && (('module' !== $rule && preg_match('/\{id\}/', $root)) || 'module' == $rule)) {
            return $root;
        }

        return $default;
    }

    public function seo_url_schema_configs()
    {
        return [
            'category_rule' => [
                'root_name' => 'ETS_AWU_ROOT_URL_CATEGORY_RULE',
                'old_name' => 'ETS_AWU_OLD_URL_CATEGORY_RULE',
                'name' => 'ETS_AWU_URL_CATEGORY_RULE',
                'no_id' => 'ETS_AWU_URL_NOID_CATEGORY_RULE',
                'default' => '{id}-{rewrite}',
            ],
            'supplier_rule' => [
                'root_name' => 'ETS_AWU_ROOT_URL_SUPPLIER_RULE',
                'old_name' => 'ETS_AWU_OLD_URL_SUPPLIER_RULE',
                'name' => 'ETS_AWU_URL_SUPPLIER_RULE',
                'no_id' => 'ETS_AWU_URL_NOID_SUPPLIER_RULE',
                'default' => 'supplier/{id}-{rewrite}',
            ],
            'manufacturer_rule' => [
                'root_name' => 'ETS_AWU_ROOT_URL_MANUF_RULE',
                'old_name' => 'ETS_AWU_OLD_URL_MANUF_RULE',
                'name' => 'ETS_AWU_URL_MANUF_RULE',
                'no_id' => 'ETS_AWU_URL_NOID_MANUF_RULE',
                'default' => 'brand/{id}-{rewrite}',
            ],
            'cms_rule' => [
                'root_name' => 'ETS_AWU_ROOT_URL_CMS_RULE',
                'old_name' => 'ETS_AWU_OLD_URL_CMS_RULE',
                'name' => 'ETS_AWU_URL_CMS_RULE',
                'no_id' => 'ETS_AWU_URL_NOID_CMS_RULE',
                'default' => 'content/{id}-{rewrite}',
            ],
            'cms_category_rule' => [
                'root_name' => 'ETS_AWU_ROOT_URL_CMS_CATEGORY_RULE',
                'old_name' => 'ETS_AWU_OLD_URL_CMS_CATEGORY_RULE',
                'name' => 'ETS_AWU_URL_CMS_CATEGORY_RULE',
                'no_id' => 'ETS_AWU_URL_NOID_CMS_CATEGORY_RULE',
                'default' => 'content/category/{id}-{rewrite}',
            ],
            'module' => [
                'root_name' => 'ETS_AWU_ROOT_URL_MODULE_RULE',
                'old_name' => 'ETS_AWU_OLD_URL_MODULE_RULE',
                'name' => 'ETS_AWU_URL_MODULE_RULE',
                'no_id' => 'ETS_AWU_URL_NOID_MODULE_RULE',
                'default' => 'module/{module}{/:controller}',
            ],
            'product_rule' => [
                'root_name' => 'ETS_AWU_ROOT_URL_PRODUCT_RULE',
                'old_name' => 'ETS_AWU_OLD_URL_PRODUCT_RULE',
                'name' => 'ETS_AWU_URL_PRODUCT_RULE',
                'no_id' => 'ETS_AWU_URL_NOID_PRODUCT_RULE',
                'default' => $this->is17 ? '{category:/}{id}{-:id_product_attribute}-{rewrite}{-:ean13}.html' : '{category:/}{id}-{rewrite}{-:ean13}.html',
            ],
            'layered_rule' => [
                'root_name' => 'ETS_AWU_ROOT_URL_LAYERED_RULE',
                'old_name' => 'ETS_AWU_OLD_URL_LAYERED_RULE',
                'name' => 'ETS_AWU_URL_LAYERED_RULE',
                'no_id' => 'ETS_AWU_URL_NOID_LAYERED_RULE',
                'default' => '{id}-{rewrite}{/:selected_filters}',
            ],
        ];
    }

    public function seo_advanced($type, $id, $context)
    {
        $seo_data = null;
        $config_allow_search = '';
        $indexLabel = $this->l('Allow search engines to show this Post in search results?');
        $followLabel = $this->l('Should search engines follow links on this Product?');
        switch ($type) {
            case 'product':
                $indexLabel = $this->l('Allow search engines to show this Product page in search results?');
                $followLabel = $this->l('Should search engines follow links on this Product?');
                $config_allow_search = (int) Configuration::get('ETS_AWU_PROD_SHOW_IN_SEARCH_RESULT');
                if ((int) $id) {
                    $seo_data = EtsAwuProduct::getSeoProduct($id, $context);
                }

                break;
            case 'cms':
                $indexLabel = $this->l('Allow search engines to show this CMS page in search results?');
                $followLabel = $this->l('Should search engines follow links on this CMS?');
                $config_allow_search = (int) Configuration::get('ETS_AWU_CMS_SHOW_IN_SEARCH_RESULT');
                if ((int) $id) {
                    $seo_data = EtsAwuCms::getSeoCms($id, $context);
                }
                break;
            case 'meta':
                $indexLabel = $this->l('Allow search engines to show this Meta page in search results?');
                $followLabel = $this->l('Should search engines follow links on this Meta?');
                $config_allow_search = (int) Configuration::get('ETS_AWU_META_SHOW_IN_SEARCH_RESULT');
                if ((int) $id) {
                    $seo_data = EtsAwuMeta::getSeoMeta($id, $context);
                }
                break;
            case 'category':
                $indexLabel = $this->l('Allow search engines to show this Category page in search results?');
                $followLabel = $this->l('Should search engines follow links on this Category?');
                $config_allow_search = (int) Configuration::get('ETS_AWU_CATEGORY_SHOW_IN_SEARCH_RESULT');
                if ((int) $id) {
                    $seo_data = EtsAwuCategory::getSeoCategory($id, $context);
                }
                break;
            case 'cms_category':
                $indexLabel = $this->l('Allow search engines to show this CMS category page in search results?');
                $followLabel = $this->l('Should search engines follow links on this CMS category?');
                $config_allow_search = (int) Configuration::get('ETS_AWU_CMS_CATE_SHOW_IN_SEARCH_RESULT');
                if ((int) $id) {
                    $seo_data = EtsAwuCmsCategory::getSeoCmsCategory($id, $context);
                }
                break;
            case 'manufacturer':
                $indexLabel = $this->l('Allow search engines to show this Brand page in search results?');
                $followLabel = $this->l('Should search engines follow links on this Brand?');
                $config_allow_search = (int) Configuration::get('ETS_AWU_MANUFACTURER_SHOW_IN_SEARCH_RESULT');
                if ((int) $id) {
                    $seo_data = EtsAwuManufacturer::getSeoManufacturer($id, $context);
                }
                break;
            case 'supplier':
                $indexLabel = $this->l('Allow search engines to show this Supplier page in search results?');
                $followLabel = $this->l('Should search engines follow links on this Supplier?');
                $config_allow_search = (int) Configuration::get('ETS_AWU_SUPPLIER_SHOW_IN_SEARCH_RESULT');
                if ((int) $id) {
                    $seo_data = EtsAwuSupplier::getSeoSupplier($id, $context);
                }
                break;
        }
        $data = [
            'allow_search' => [],
            'allow_flw_link' => [],
            'meta_robots_adv' => [],
            'canonical_url' => [],
        ];
        if ($seo_data) {
            foreach ($seo_data as $seo) {
                foreach ($data as $key => $value) {
                    if (is_array($value)) {
                        $data[$key][$seo['id_lang']] = $seo[$key];
                    } elseif ('meta_robots_adv' == $key) {
                        $data[$key][$seo['id_lang']] = '';
                    }
                }
            }
        }

        $indexOptions = [
            [
                'label' => $this->l('Yes'),
                'value' => 1,
            ],
            [
                'label' => $this->l('No'),
                'value' => 0,
            ],
        ];
        $advanceds = [
            'allow_search' => [
                'label' => $indexLabel,
                'id' => 'ets_awu_allow_search_engine_show_post',
                'type' => 'select',
                'config_value' => $config_allow_search,
                'selected' => $data['allow_search'],
                'link_default' => $this->context->link->getAdminLink('AdminEtsSeoSearchAppearanceContentType', true),
                'options' => $indexOptions,
            ],
            'allow_flw_link' => [
                'label' => $followLabel,
                'id' => 'ets_awu_allow_search_engine_follow_links',
                'type' => 'radio',
                'checked' => $data['allow_flw_link'],
                'options' => [
                    [
                        'label' => $this->l('Yes'),
                        'value' => 1,
                        'id' => 'ets_awu_allow_search_engine_follow_links_yes',
                    ],
                    [
                        'label' => $this->l('No'),
                        'value' => 0,
                        'id' => 'ets_awu_allow_search_engine_follow_links_no',
                    ],
                ],
            ],
            'meta_robots_adv' => [
                'label' => $this->l('Meta robots advanced'),
                'id' => 'ets_awu_meta_robots_advanced',
                'type' => 'select2',
                'options' => [
                    [
                        'label' => $this->l('Site-wide default'),
                        'value' => 'default',
                    ],
                    [
                        'label' => $this->l('None'),
                        'value' => 'none',
                    ],
                    [
                        'label' => $this->l('No Image index'),
                        'value' => 'noimageindex',
                    ],
                    [
                        'label' => $this->l('No Archive'),
                        'value' => 'noarchive',
                    ],
                    [
                        'label' => $this->l('No Snippet'),
                        'value' => 'nosnippet',
                    ],
                ],
                'selected' => $data['meta_robots_adv'],
                'multiple' => true,
                'desc' => $this->l('Advanced meta robots settings for this page.'),
            ],
            'canonical_url' => [
                'label' => $this->l('Canonical URL'),
                'id' => 'ets_awu_canonical_url',
                'type' => 'input_text',
                'value' => $data['canonical_url'],
                'desc' => $this->l('The canonical URL that this page should point to. Leave empty to default to current page link. Cross domain canonical (Opens in a new browser tab) supported too..'),
            ],
        ];
        if (!$this->getModule()->is17) {
            unset($advanceds['meta_robots_adv']);
        }

        return $advanceds;
    }

    public static function getTextLang($text, $lang, $file_name = '')
    {
        $moduleName = 'ets_awesomeurl';
        $text2 = preg_replace("/\\\*'/", "\'", $text);
        if (is_array($lang)) {
            $iso_code = $lang['iso_code'];
        } elseif (is_object($lang)) {
            $iso_code = $lang->iso_code;
        } else {
            $language = new Language($lang);
            $iso_code = $language->iso_code;
        }
        $modulePath = rtrim(_PS_MODULE_DIR_, '/') . '/' . $moduleName;
        $fileTransDir = $modulePath . '/translations/' . $iso_code . '.php';
        if (!@file_exists($fileTransDir)) {
            return '';
        }
        $fileContent = Tools::file_get_contents($fileTransDir);
        $strMd5 = md5($text2);
        $keyMd5 = '<{' . $moduleName . '}prestashop>' . ($file_name ? Tools::strtolower($file_name) : $moduleName) . '_' . $strMd5;
        preg_match('/(\$_MODULE\[\'' . preg_quote($keyMd5) . '\'\]\s*=\s*\')(.*)(\';)/', $fileContent, $matches);
        if ($matches && isset($matches[2])) {
            return $matches[2];
        }

        return '';
    }

    /**
     * @return \Ets_awesomeurl|\Module
     */
    public function getModule()
    {
        if (!$this->module instanceof \Ets_awesomeurl) {
            $this->module = \Module::getInstanceByName('ets_awesomeurl');
        }

        return $this->module;
    }

    /**
     * @return string[]
     */
    public function getHooks()
    {
        return $this->hooks;
    }

    /**
     * Get formatted name.
     *
     * @param string $name
     *
     * @return string
     */
    public static function getFormattedName($name)
    {
        $themeName = Context::getContext()->shop->theme_name;
        $nameWithoutThemeName = str_replace(['_' . $themeName, $themeName . '_'], '', $name);

        // check if the theme name is already in $name if yes only return $name
        if ($themeName !== null && strstr($name, $themeName) && self::getByNameNType($name)) {
            return $name;
        }

        if (self::getByNameNType($nameWithoutThemeName . '_' . $themeName)) {
            return $nameWithoutThemeName . '_' . $themeName;
        }

        if (self::getByNameNType($themeName . '_' . $nameWithoutThemeName)) {
            return $themeName . '_' . $nameWithoutThemeName;
        }

        return $nameWithoutThemeName . '_default';
    }

    /**
     * Finds image type definition by name and type.
     *
     * @param string $name
     * @param null $type
     * @param int $order
     *
     * @return false|mixed
     *
     * @throws \PrestaShopDatabaseException
     */
    public static function getByNameNType($name, $type = null, $order = 0)
    {
        static $is_passed = false;
        $cacheKey = $name . '_' . $type . '_' . $order;
        if (!isset(self::$images_types_name_cache[$cacheKey]) && !$is_passed) {
            $results = Db::getInstance()->executeS('SELECT * FROM `' . _DB_PREFIX_ . 'image_type`');

            $types = ['products', 'categories', 'manufacturers', 'suppliers', 'stores'];
            $total = count($types);

            foreach ($results as $result) {
                foreach ($result as $value) {
                    for ($i = 0; $i < $total; ++$i) {
                        self::$images_types_name_cache[$result['name'] . '_' . $types[$i] . '_' . $value] = $result;
                    }
                }
            }

            $is_passed = true;
        }

        $return = false;
        if (isset(self::$images_types_name_cache[$cacheKey])) {
            $return = self::$images_types_name_cache[$cacheKey];
        }

        return $return;
    }

    public static function checkEnableOtherShop($id_module)
    {
        $sql = 'SELECT * FROM `' . _DB_PREFIX_ . 'module_shop` WHERE `id_module` = ' . (int) $id_module . ' AND `id_shop` NOT IN(' . implode(', ', Shop::getContextListShopID()) . ')';

        return Db::getInstance()->executeS($sql);
    }

    public static function activeTab($module_name)
    {
        return Db::getInstance()->execute('UPDATE `' . _DB_PREFIX_ . 'tab` SET enabled=1 where module ="' . pSQL($module_name) . '"');
    }
}
