<?php
/**
 * Copyright ETS Software Technology Co., Ltd
 *
 * NOTICE OF LICENSE
 *
 * This file is not open source! Each license that you purchased is only available for 1 website only.
 * If you want to use this file on more websites (or projects), you need to purchase additional licenses.
 * You are not allowed to redistribute, resell, lease, license, sub-license or offer our resources to any third party.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future.
 *
 * @author ETS Software Technology Co., Ltd
 * @copyright  ETS Software Technology Co., Ltd
 * @license    Valid for 1 website (or project) for each purchase of license
 */
if (!defined('_PS_VERSION_')) {
    exit;
}
class EtsAwuRedirect extends ObjectModel
{
    /**
     * @var int
     */
    public $id_ets_awu_redirect;

    /**
     * @var string
     */
    public $url;

    /**
     * @var string
     */
    public $target;

    /**
     * @var string
     */
    public $type;

    /**
     * @var bool
     */
    public $active;

    /**
     * @var int
     */
    public $id_shop;

    public static $definition = [
        'table' => 'ets_awu_redirect',
        'primary' => 'id_ets_awu_redirect',
        'multilang_shop' => false,
        'fields' => [
            'id_ets_awu_redirect' => [
                'type' => self::TYPE_INT,
                'validate' => 'isUnsignedInt',
            ],
            'url' => [
                'type' => self::TYPE_STRING,
                'validate' => 'isString',
            ],
            'target' => [
                'type' => self::TYPE_STRING,
                'validate' => 'isString',
            ],
            'type' => [
                'type' => self::TYPE_STRING,
                'validate' => 'isString',
            ],
            'active' => [
                'type' => self::TYPE_BOOL,
                'validate' => 'isBool',
            ],

            'id_shop' => [
                'type' => self::TYPE_INT,
                'validate' => 'isUnsignedInt',
            ],
        ],
    ];

    /**
     * @param string $url
     *
     * @return string
     */
    private static function removeShopBaseUrl($url)
    {
        $shopBase = Context::getContext()->shop->getBaseURL(true, true);
        $url = str_replace($shopBase, '', $url);

        return ltrim($url, '/');
    }

    /**
     * @param string $current_url
     * @param \Context|\ContextCore|null $context
     * @param bool $active
     *
     * @return array|false
     */
    public static function getTypeUrlRedirect($current_url, $context = null, $active = false)
    {
        if (!$context) {
            $context = Context::getContext();
        }
        $url = trim(self::removeShopBaseUrl($current_url));
        $where_active = $active ? ' AND active = 1' : '';
        if (false !== strpos($url, '/*')) {
            return Db::getInstance()->getRow('SELECT `type`, `target` FROM `' . _DB_PREFIX_ . "ets_awu_redirect` WHERE '" . pSQL($url) . "' REGEXP (REPLACE(`url`, '*', '(.*)')) " . pSQL($where_active) . ' AND `id_shop`=' . (int) $context->shop->id);
        }

        return Db::getInstance()->getRow('SELECT `type`, `target` FROM `' . _DB_PREFIX_ . "ets_awu_redirect` WHERE `url` = '" . pSQL($url) . "'" . pSQL($where_active) . ' AND `id_shop`=' . (int) $context->shop->id);
    }
}
