<?php
/**
 * Copyright ETS Software Technology Co., Ltd
 *
 * NOTICE OF LICENSE
 *
 * This file is not open source! Each license that you purchased is only available for 1 website only.
 * If you want to use this file on more websites (or projects), you need to purchase additional licenses.
 * You are not allowed to redistribute, resell, lease, license, sub-license or offer our resources to any third party.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future.
 *
 * @author ETS Software Technology Co., Ltd
 * @copyright  ETS Software Technology Co., Ltd
 * @license    Valid for 1 website (or project) for each purchase of license
 */
if (!defined('_PS_VERSION_')) {
    exit;
}
class Ets_Awu_Sitemap
{
    public $context;
    public $request_uri;
    /**
     * @var int
     */
    private $currentLangId;
    /**
     * @var string
     */
    private $currentPageType;

    public function __construct($request_uri, $context = null)
    {
        $this->request_uri = $request_uri;
        if (!$context) {
            $this->context = Context::getContext();
        }
    }

    public function sitemap()
    {
        $xml = '';
        if ((int) Configuration::get('ETS_AWU_ENABLE_XML_SITEMAP')) {
            $isoCode = $this->getLangFromUrl(true);
            $page_type = $this->getPageType();
            if ($isoCode && Language::isMultiLanguageActivated()) {
                $xml = $this->sitemapPage($isoCode, $page_type);
            } else {
                $languages = Language::getLanguages(true, $this->context->shop->id);
                if (count($languages) > 1 && Language::isMultiLanguageActivated()) {
                    $xml = $this->sitemapLang($languages);
                } else {
                    $xml = $this->sitemapPage($this->context->language->iso_code, $page_type);
                }
            }
        }

        if (ob_get_length() > 0) {
            ob_end_clean();
        }
        header('Content-Type: application/xml; charset=UTF-8');
        mb_internal_encoding('UTF-8');
        exit($xml);
    }

    /**
     * getLinkSitemap.
     *
     * @param array $params
     *
     * @return string
     */
    protected function getLinkSitemap($params = [])
    {
        if ('blog' == $params['page_type']) {
            if (isset($params['lang']) && $params['lang']) {
                $idLang = (int) Language::getIdByIso($params['lang']);
            } else {
                $idLang = (int) Context::getContext()->language->id;
            }

            return Context::getContext()->link->getModuleLink('ybc_blog', 'sitemap', [], null, $idLang);
        }
        $linkObj = Context::getContext()->link;
        $link = $linkObj->getBaseLink();
        if (isset($params['lang']) && $params['lang'] && Language::isMultiLanguageActivated()) {
            $link .= $params['lang'] . '/';
        }
        if (!isset($params['prependSitemap'])) {
            $meta = Meta::getMetaByPage('module-ets_awesomeurl-sitemap', $this->currentLangId);
            $link .= $meta ? $meta['url_rewrite'] : 'sitemap';
        }
        if (isset($params['page_type']) && $params['page_type']) {
            if (isset($params['page']) && (int) $params['page']) {
                return rtrim($link, '/') . '/' . $params['page_type'] . '/' . (int) $params['page'] . '.xml';
            }

            return rtrim($link, '/') . '/' . $params['page_type'] . '.xml';
        }

        return trim($link . '.xml');
    }

    /**
     * @param string $isoCode
     *
     * @return string
     *
     * @throws \PrestaShopException
     */
    private function listIndexByLang($isoCode)
    {
        static $languages;
        static $listItemsDisplayedSitemap;
        if (!$languages) {
            $languages = Language::getLanguages();
        }
        if (!$listItemsDisplayedSitemap) {
            $listItemsDisplayedSitemap = explode(',', (string) Configuration::get('ETS_AWU_SITEMAP_OPTION'));
        }
        $id_lang = Language::getIdByIso($isoCode);
        $xml = '';
        if (in_array('product', $listItemsDisplayedSitemap, true)) {
            $xml .= $this->getSitemapProduct($id_lang);
        }

        if (in_array('category', $listItemsDisplayedSitemap, true)) {
            $xml .= '<sitemap>';
            $xml .= '<loc><![CDATA[' . $this->getLinkSitemap(['lang' => $isoCode, 'page_type' => 'category']) . ']]></loc>';
            $xml .= '</sitemap>';
        }
        if (in_array('cms', $listItemsDisplayedSitemap, true) && CMS::getCMSPages($id_lang, null, true, $this->context->shop->id)) {
            $xml .= '<sitemap>';
            $xml .= '<loc><![CDATA[' . $this->getLinkSitemap(['lang' => $isoCode, 'page_type' => 'cms']) . ']]></loc>';
            $xml .= '</sitemap>';
        }
        if (in_array('cms_category', $listItemsDisplayedSitemap, true) && CMSCategory::getSimpleCategories($id_lang)) {
            $xml .= '<sitemap>';
            $xml .= '<loc><![CDATA[' . $this->getLinkSitemap(['lang' => $isoCode, 'page_type' => 'cms_category']) . ']]></loc>';
            $xml .= '</sitemap>';
        }
        if (in_array('meta', $listItemsDisplayedSitemap, true) && Meta::getMetasByIdLang($id_lang)) {
            $xml .= '<sitemap>';
            $xml .= '<loc><![CDATA[' . $this->getLinkSitemap(['lang' => $isoCode, 'page_type' => 'meta']) . ']]></loc>';
            $xml .= '</sitemap>';
        }
        if (in_array('manufacturer', $listItemsDisplayedSitemap, true) && Manufacturer::getManufacturers(false, $id_lang, true)) {
            $xml .= '<sitemap>';
            $xml .= '<loc><![CDATA[' . $this->getLinkSitemap(['lang' => $isoCode, 'page_type' => 'manufacturer']) . ']]></loc>';
            $xml .= '</sitemap>';
        }
        if (in_array('supplier', $listItemsDisplayedSitemap, true) && Supplier::getSuppliers(false, $id_lang, true)) {
            $xml .= '<sitemap>';
            $xml .= '<loc><![CDATA[' . $this->getLinkSitemap(['lang' => $isoCode, 'page_type' => 'supplier']) . ']]></loc>';
            $xml .= '</sitemap>';
        }
        if (in_array('blog', $listItemsDisplayedSitemap, true) && Module::isEnabled('ybc_blog')) {
            if (count($languages) < 2) {
                $xml .= '<sitemap>';
                $xml .= '<loc><![CDATA[' . $this->getLinkSitemap(['page_type' => 'blog_sitemap', 'prependSitemap' => false]) . ']]></loc>';
                $xml .= '</sitemap>';
            } else {
                $xml .= '<sitemap>';
                $xml .= '<loc><![CDATA[' . $this->getLinkSitemap(['lang' => $isoCode, 'page_type' => 'blog_sitemap', 'prependSitemap' => false]) . ']]></loc>';
                $xml .= '</sitemap>';
            }
        }

        return $xml;
    }

    /**
     * sitemapLang.
     *
     * @param array $languages
     *
     * @return string
     */
    protected function sitemapLang($languages)
    {
        $xml = '<?xml version="1.0" encoding="UTF-8"?>';
        $xml .= '<sitemapindex xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">';
        foreach ($languages as $lang) {
            $xml .= $this->listIndexByLang($lang['iso_code']);
        }
        $xml .= '</sitemapindex>';

        return $xml;
    }

    /**
     * sitemapPage.
     *
     * @param string $isoCode
     * @param string $page_type
     *
     * @return string
     */
    protected function sitemapPage($isoCode, $page_type = null)
    {
        if ($page_type) {
            $id_lang = Language::getIdByIso($isoCode);
            switch ($page_type) {
                case 'category':
                    return $this->getSitemapCategory($id_lang);
                case 'cms':
                    return $this->getSitemapCms($id_lang);
                case 'meta':
                    return $this->getSitemapMeta($id_lang);
                case 'product':
                    return $this->getSitemapProduct($id_lang);
                case 'cms_category':
                    return $this->getSitemapCmsCategory($id_lang);
                case 'manufacturer':
                    return $this->getSitemapManufacturer($id_lang);
                case 'supplier':
                    return $this->getSitemapSupplier($id_lang);
            }
        }
        $xml = '<?xml version="1.0" encoding="UTF-8"?>';
        $xml .= '<sitemapindex xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">';

        $xml .= $this->listIndexByLang($isoCode);

        $xml .= '</sitemapindex>';

        return $xml;
    }

    protected function getSitemapCategory($id_lang)
    {
        $categories = EtsAwuCategory::getCategoriesWithoutRoot();
        $params = [];
        $link = Context::getContext()->link;
        foreach ($categories as $item) {
            $cate = new Category($item['id_category'], $id_lang);
            $dataItem = ['last_modify' => date('Y-m-d', strtotime($cate->date_upd ?: time()))];
            $dataItem['link'] = $this->addParamsToUrl($cate->getLink(Context::getContext()->link, $id_lang));
            $dataItem['image'] = [
                'link' => $link->getCatImageLink($cate->link_rewrite, $cate->id),
            ];

            $params[] = $dataItem;
        }

        return $this->renderXmlLinks($params, [
            'priority' => Configuration::get('ETS_AWU_SITEMAP_PRIORITY_CATEGORY'),
            'changefreq' => Configuration::get('ETS_AWU_SITEMAP_FREQ_CATEGORY'),
        ]);
    }

    protected function getSitemapMeta($id_lang)
    {
        $metas = Meta::getMetasByIdLang($id_lang);
        $params = [];
        foreach ($metas as $meta) {
            if (!$meta['url_rewrite'] && 'index' !== $meta['page']) {
                continue;
            }
            $dataItem = [];
            $link = Context::getContext()->link;
            $dataItem['link'] = $link->getPageLink($meta['page'], null, $id_lang);
            $params[] = $dataItem;
        }

        return $this->renderXmlLinks($params, [
            'priority' => Configuration::get('ETS_AWU_SITEMAP_PRIORITY_META'),
            'changefreq' => Configuration::get('ETS_AWU_SITEMAP_FREQ_META'),
        ]);
    }

    protected function getSitemapCms($id_lang)
    {
        $cmsPages = CMS::getCMSPages($id_lang, null, true, $this->context->shop->id);
        $links = [];
        foreach ($cmsPages as $cms) {
            $dataItem = [];
            $obj = new CMS($cms['id_cms'], $id_lang);
            $link = Context::getContext()->link;
            $dataItem['link'] = $link->getCMSLink($obj, null, null, $id_lang);
            $links[] = $dataItem;
        }

        return $this->renderXmlLinks($links, [
            'priority' => Configuration::get('ETS_AWU_SITEMAP_PRIORITY_CMS'),
            'changefreq' => Configuration::get('ETS_AWU_SITEMAP_FREQ_CMS'),
        ]);
    }

    protected function getSitemapCmsCategory($id_lang)
    {
        $cmsPages = CMSCategory::getSimpleCategories($id_lang);
        $links = [];
        foreach ($cmsPages as $cms) {
            $obj = new CMSCategory($cms['id_cms_category'], $id_lang);
            $link = Context::getContext()->link;
            $dataItem = ['last_modify' => date('Y-m-d', strtotime($obj->date_upd ?: time()))];
            $dataItem['link'] = $link->getCMSCategoryLink($obj, null, $id_lang);
            $links[] = $dataItem;
        }

        return $this->renderXmlLinks($links, [
            'priority' => Configuration::get('ETS_AWU_SITEMAP_PRIORITY_CMS_CATEGORY'),
            'changefreq' => Configuration::get('ETS_AWU_SITEMAP_FREQ_CMS_CATEGORY'),
        ]);
    }

    protected function getSitemapManufacturer($id_lang)
    {
        $manufs = Manufacturer::getManufacturers(false, $id_lang, true);

        $links = [];
        foreach ($manufs as $manuf) {
            $link = Context::getContext()->link;
            $obj = new Manufacturer($manuf['id_manufacturer'], $id_lang);
            $dataItem = ['last_modify' => date('Y-m-d', strtotime($obj->date_upd ?: time()))];
            $dataItem['link'] = $link->getManufacturerLink($obj, null, $id_lang);
            if (file_exists(_PS_ROOT_DIR_ . '/img/m/' . $obj->id . '.jpg')) {
                $dataItem['image'] = [
                    'link' => $this->context->shop->getBaseURL(true, true) . 'img/m/' . $obj->id . '.jpg',
                ];
            }
            $links[] = $dataItem;
        }

        return $this->renderXmlLinks($links, [
            'priority' => Configuration::get('ETS_AWU_SITEMAP_PRIORITY_MANUFACTURER'),
            'changefreq' => Configuration::get('ETS_AWU_SITEMAP_FREQ_MANUFACTURER'),
        ]);
    }

    protected function getSitemapSupplier($id_lang)
    {
        $suppliers = Supplier::getSuppliers(false, $id_lang, true);
        $links = [];
        foreach ($suppliers as $supplier) {
            $link = Context::getContext()->link;
            $obj = new Supplier($supplier['id_supplier'], $id_lang);
            $dataItem = ['last_modify' => date('Y-m-d', strtotime($obj->date_upd ?: time()))];
            $dataItem['link'] = $link->getSupplierLink($obj, null, $id_lang);

            if (file_exists(_PS_ROOT_DIR_ . '/img/s/' . $obj->id . '.jpg')) {
                $dataItem['image'] = [
                    'link' => $this->context->shop->getBaseURL(true, true) . 'img/s/' . $obj->id . '.jpg',
                ];
            }
            $links[] = $dataItem;
        }

        return $this->renderXmlLinks($links, [
            'priority' => Configuration::get('ETS_AWU_SITEMAP_PRIORITY_SUPPLIER'),
            'changefreq' => Configuration::get('ETS_AWU_SITEMAP_FREQ_SUPPLIER'),
        ]);
    }

    protected function getSitemapProduct($id_lang)
    {
        $per_page = 250;
        if ($limit_prod = (int) Configuration::get('ETS_AWU_PROD_SITEMAP_LIMIT')) {
            $per_page = $limit_prod;
        }
        $page = $this->getProductPage();
        if ($page) {
            $current_page = (int) Tools::getValue('page');
            if (!$current_page) {
                $current_page = 1;
            }
            $start = ($current_page - 1) * $per_page;
            $products = Db::getInstance()->executeS('
                    SELECT p.id_product FROM `' . _DB_PREFIX_ . 'product` p 
                    LEFT JOIN `' . _DB_PREFIX_ . 'product_shop` ps ON p.id_product=ps.id_product AND ps.id_shop=' . (int) $this->context->shop->id . " 
                    WHERE ps.`visibility` IN ('both', 'catalog') AND ps.`active`=1
                    ORDER BY p.id_product DESC
                    LIMIT " . (int) $start . ',' . (int) $per_page);
            $links = [];
            foreach ($products as $item) {
                $dataItem = [];
                $link = Context::getContext()->link;
                $product = new Product((int) $item['id_product'], false, $id_lang);

                $dataItem['link'] = $product->getLink($this->context);
                $images = Product::getCover($product->id);
                if ($images && isset($images['id_image']) && (int) $images['id_image']) {
                    $caption = null;
                    $img = new Image($images['id_image'], $id_lang);
                    if ($img && isset($img->legend)) {
                        $caption = $img->legend;
                    }
                    $dataItem['image'] = [
                        'link' => $link->getImageLink($product->link_rewrite, isset($images['id_image']) ? $images['id_image'] : '', EtsAwuDefine::getFormattedName('home')),
                        'caption' => $caption,
                        'title' => $product->name,
                    ];
                }
                $dataItem['last_modify'] = date('Y-m-d', strtotime($product->date_upd ?: time()));
                $links[] = $dataItem;
            }

            return $this->renderXmlLinks($links, [
                'priority' => Configuration::get('ETS_AWU_SITEMAP_PRIORITY_PRODUCT'),
                'changefreq' => Configuration::get('ETS_AWU_SITEMAP_FREQ_PRODUCT'),
            ]);
        }
        $total_product = EtsAwuProduct::getTotalProduct(true, $id_lang);
        $isoCode = Language::getIsoById($id_lang);
        $total_page = ceil($total_product / $per_page);
        if ($total_page) {
            for ($p = 1; $p <= $total_page; ++$p) {
                $paramPage = ['lang' => $isoCode, 'page_type' => 'product', 'page' => $p];
                $links[] = $this->getLinkSitemap($paramPage);
            }

            return $this->renderXmlPages($links);
        }
    }

    /**
     * renderXmlPage.
     *
     * @param array $links
     * @param array $params
     *
     * @return string
     */
    protected function renderXmlLinks($links, $params = [])
    {
        $xml = '<?xml version="1.0" encoding="UTF-8"?>';
        $xml .= '<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9" xmlns:image="http://www.google.com/schemas/sitemap-image/1.1">';

        if ($links) {
            foreach ($links as $link) {
                $xml .= '<url>';
                $xml .= '<loc><![CDATA[' . $link['link'] . ']]></loc>';
                $xml .= '<changefreq>' . (isset($params['changefreq']) && $params['changefreq'] ? $params['changefreq'] : 'weekly') . '</changefreq>';
                $xml .= '<priority>' . (isset($params['priority']) && '' !== $params['priority'] ? number_format($params['priority'], 1, '.', '') : 1.0) . '</priority>';
                if (isset($link['image']) && !empty($link['image'])) {
                    $xml .= '<image:image>';
                    $xml .= '<image:loc><![CDATA[' . $link['image']['link'] . ']]></image:loc>';
                    if (isset($link['image']['caption']) && $link['image']['caption'] && strip_tags($link['image']['caption'])) {
                        $xml .= '<image:caption><![CDATA[' . strip_tags($link['image']['caption']) . ']]></image:caption>';
                    }
                    if (isset($link['image']['title']) && $link['image']['title'] && strip_tags($link['image']['title'])) {
                        $xml .= '<image:title><![CDATA[' . strip_tags($link['image']['title']) . ']]></image:title>';
                    }
                    $xml .= '</image:image>';
                }
                if (isset($link['last_modify']) && Validate::isDate($link['last_modify'])) {
                    $xml .= '<lastmod>' . $link['last_modify'] . '</lastmod>';
                }
                $xml .= '</url>';
            }
        }
        $xml .= '</urlset>';

        return $xml;
    }

    protected function renderXmlPages($pages)
    {
        $xml = '';
        foreach ($pages as $page) {
            $xml .= '<sitemap>';
            $xml .= '<loc><![CDATA[' . $page . ']]></loc>';
            $xml .= '</sitemap>';
        }

        return $xml;
    }

    /**
     * addParamsToUrl.
     *
     * @param string $link
     * @param array $params
     *
     * @return string
     */
    protected function addParamsToUrl($link, $params = [])
    {
        $count = 0;
        foreach ($params as $k => $p) {
            if (0 == $count && false !== Tools::strpos($link, '?')) {
                $link = $link . '?' . $k . '=' . $p;
            } else {
                $link = $link . '&' . $k . '=' . $p;
            }
        }

        return $link;
    }

    public function getLangFromUrl($getIsoCode = false)
    {
        // Get request uri (HTTP_X_REWRITE_URL is used by IIS)
        if (isset($_SERVER['REQUEST_URI'])) {
            $requestUri = $_SERVER['REQUEST_URI'];
        } elseif (isset($_SERVER['HTTP_X_REWRITE_URL'])) {
            $requestUri = $_SERVER['HTTP_X_REWRITE_URL'];
        }
        $requestUri = rawurldecode($requestUri);

        if (isset(Context::getContext()->shop) && is_object(Context::getContext()->shop)) {
            $requestUri = preg_replace(
                '#^' . preg_quote(Context::getContext()->shop->getBaseURI(), '#') . '#i',
                '/',
                $requestUri
            );
        }

        // If there are several languages, get language from uri
        if (Language::isMultiLanguageActivated()) {
            if (preg_match('#^/([a-z]{2})(?:/.*)?$#', $requestUri, $m)) {
                $isoCode = $m[1];
                if ($isoCode) {
                    $id_lang = Language::getIdByIso($isoCode);
                    $this->currentLangId = (int) $id_lang;
                    if ($id_lang) {
                        if ($getIsoCode) {
                            return $isoCode;
                        }

                        return (int) $id_lang;
                    }

                    return false;
                }
            }
        }

        return 0;
    }

    /**
     * @return string|null
     */
    public function getPageType()
    {
        $types = [
            'product',
            'category',
            'cms',
            'cms_category',
            'meta',
            'manufacturer',
            'supplier',
        ];
        $page_type = null;
        foreach ($types as $type) {
            $meta = Meta::getMetaByPage('module-ets_seo-sitemap', $this->currentLangId);
            $prefix = $meta ? $meta['url_rewrite'] : 'sitemap';
            $pattern = sprintf('/%s\/%s(\/(\d+)|)\.xml/', $prefix, $type);
            if (preg_match($pattern, $this->request_uri)) {
                $this->currentPageType = $type;
                $page_type = $type;
            }
        }

        return $page_type;
    }

    public function getProductPage()
    {
        if (preg_match("/sitemap\/product\/\d+\.xml$/", $this->request_uri)) {
            $uri = $this->request_uri;
            if (preg_match('/\?/', $this->request_uri)) {
                $uri = explode('?', $this->request_uri)[0];
            }
            $uri = explode('/', $uri);
            $uri = end($uri);

            return (int) str_replace('.xml', '', $uri);
        }

        return false;
    }
}
