<?php
/**
 * Copyright ETS Software Technology Co., Ltd
 *
 * NOTICE OF LICENSE
 *
 * This file is not open source! Each license that you purchased is only available for 1 website only.
 * If you want to use this file on more websites (or projects), you need to purchase additional licenses.
 * You are not allowed to redistribute, resell, lease, license, sub-license or offer our resources to any third party.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future.
 *
 * @author ETS Software Technology Co., Ltd
 * @copyright  ETS Software Technology Co., Ltd
 * @license    Valid for 1 website (or project) for each purchase of license
 */
if (!defined('_PS_VERSION_')) {
    exit;
}
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Trait EtsAwuRequestControllerTrait
 */
trait EtsAwuRequestControllerTrait
{
    /**
     * @var ContainerInterface
     */
    private static $_sfContainer;
    /**
     * @var \Symfony\Component\HttpFoundation\Request
     */
    private static $_sfRequest;

    /**
     * @return \Symfony\Component\DependencyInjection\Container|\PrestaShop\PrestaShop\Adapter\Container\LegacyContainer|ContainerInterface
     */
    public static function getSfContainer()
    {
        if (!self::$_sfContainer instanceof ContainerInterface) {
            if (!class_exists('\PrestaShop\PrestaShop\Adapter\SymfonyContainer')) {
                $kernel = null;
                try {
                    $env = (true == _PS_MODE_DEV_) ? 'dev' : 'prod';
                    $kernel = new AppKernel($env, _PS_MODE_DEV_);
                    if (method_exists($kernel, 'loadClassCache')) { // This method has been deleted in Symfony 4.x
                        $kernel->loadClassCache();
                    }
                    $kernel->boot();
                    self::$_sfContainer = $kernel->getContainer();

                    return self::$_sfContainer;
                } catch (Exception $ex) {
                    return null;
                }
            }
            self::$_sfContainer = call_user_func(['\PrestaShop\PrestaShop\Adapter\SymfonyContainer', 'getInstance']);
        }

        return self::$_sfContainer;
    }

    /**
     * @return \Symfony\Component\HttpFoundation\Request|null
     */
    public static function getRequestContainer()
    {
        if (self::$_sfRequest instanceof \Symfony\Component\HttpFoundation\Request) {
            return self::$_sfRequest;
        }
        if ($sfContainer = self::getSfContainer()) {
            self::$_sfRequest = $sfContainer->get('request_stack')->getCurrentRequest();

            return self::$_sfRequest;
        }

        return null;
    }

    /**
     * @return bool
     */
    private function isMetaController()
    {
        if ($request = self::getRequestContainer()) {
            $route = $request->get('_route');
            if ($route && (0 === strpos($route, 'admin_metas_'))) {
                return true;
            }
        }

        return 'AdminMeta' === Tools::getValue('controller');
    }

    /**
     * @return bool
     */
    private function isCmsCategoryPage()
    {
        $request = self::getRequestContainer();
        if ($request) {
            if ('admin_cms_pages_category_edit' == $request->get('_route') || 'admin_cms_pages_category_create' == $request->get('_route')) {
                return true;
            }
        } else {
            if (Tools::getIsset('addcms_category') || Tools::getIsset('updatecms_category')) {
                return true;
            }
        }

        return false;
    }
}
