<?php
/**
 * Copyright ETS Software Technology Co., Ltd
 *
 * NOTICE OF LICENSE
 *
 * This file is not open source! Each license that you purchased is only available for 1 website only.
 * If you want to use this file on more websites (or projects), you need to purchase additional licenses.
 * You are not allowed to redistribute, resell, lease, license, sub-license or offer our resources to any third party.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future.
 *
 * @author ETS Software Technology Co., Ltd
 * @copyright  ETS Software Technology Co., Ltd
 * @license    Valid for 1 website (or project) for each purchase of license
 */
if (!defined('_PS_VERSION_')) {
    exit;
}
require_once __DIR__ . '/classes/EtsAwuDefine.php';
require_once __DIR__ . '/classes/EtsAwuRedirect.php';
require_once __DIR__ . '/classes/EtsAwuUpdating.php';
require_once __DIR__ . '/classes/Ets_Awu_Sitemap.php';
require_once __DIR__ . '/classes/EtsAwuCategory.php';
require_once __DIR__ . '/classes/EtsAwuProduct.php';
require_once __DIR__ . '/classes/EtsAwuSetting.php';
require_once __DIR__ . '/classes/EtsAwuManufacturer.php';
require_once __DIR__ . '/classes/EtsAwuSupplier.php';
require_once __DIR__ . '/classes/EtsAwuCms.php';
require_once __DIR__ . '/classes/EtsAwuCmsCategory.php';
require_once __DIR__ . '/classes/EtsAwuMeta.php';
require_once __DIR__ . '/classes/traits/EtsAwuRequestControllerTrait.php';

if (!class_exists('EtsUrlHelper')) {
    require_once __DIR__ . '/classes/utils/EtsUrlHelper.php';
}

/**
 * Class Ets_awesomeurl.
 *
 * @property \Context|\ContextCore $context
 *
 * @mixin \ModuleCore
 */
class Ets_awesomeurl extends Module
{
    use EtsAwuRequestControllerTrait;
    public $is17;
    public $is176;
    public $is175;
    public $is178;
    public $is15 = false;
    public $is16;
    protected $is_configurable = true;

    /**
     * @var string[]
     */
    private $_excludedHosts = [
        'localhost',
        '*.zuko.pro',
        '*.presta-demos.com',
        '*.test',
    ];

    public function __construct()
    {
        $this->name = 'ets_awesomeurl';
        $this->tab = 'seo';
        $this->version = '1.2.8';
        $this->author = 'PrestaHero';
        $this->bootstrap = true;
        parent::__construct();
        $this->displayName = $this->l('Awesome URL');
        $this->description = $this->l('Create SEO-friendly URLs for better SEO & customer experience');
$this->refs = 'https://prestahero.com/';
        $this->ps_versions_compliancy = ['min' => '1.6', 'max' => _PS_VERSION_];
        $this->module_key = 'e0a0c4eaa28f502117106a5f4a465f51';
        $this->is17 = version_compare('1.7.0.0', _PS_VERSION_, '<=');
        $this->is176 = version_compare(_PS_VERSION_, '1.7.6.0', '>=');
        $this->is175 = version_compare(_PS_VERSION_, '1.7.5.0', '>=');
        $this->is178 = version_compare(_PS_VERSION_, '1.7.8.0', '>=');
        $this->is16 = false;
        if (version_compare(_PS_VERSION_, '1.7', '<')) {
            $this->is16 = true;
        }
    }

    public function install()
    {
        if (self::isInstalled('ets_seo')) {
            throw new PrestaShopException($this->l('The module ets_seo has been installed'));
        }
        if (self::isInstalled('etsdynamicsitemap')) {
            throw new PrestaShopException($this->l('The module etsdynamicsitemap has been installed'));
        }
        if (Module::isEnabled('prettyurls')) {
            throw new \PrestaShopException($this->l('Pretty Url module is installed & enabled. You cannot enable this module'));
        }
        $etsDef = EtsAwuDefine::getInstance();

        return parent::install()
            && $this->_registerHooks()
            && EtsAwuSetting::_installTabs()
            && $etsDef->installDb()
            && $this->setDefaultConfig()
            && $this->setRootSeoUrlConfig()
            && $this->_installOverried()
            && $this->setSitemap()
            && $this->copySymfonyServices();
    }

    /**
     * @return bool
     */
    private function _registerHooks()
    {
        $hooks = EtsAwuDefine::getInstance()->getHooks();
        foreach ($hooks as $hook) {
            if (!$this->registerHook($hook)) {
                return false;
            }
        }
        if ($this->is178) {
            return $this->registerHook('actionAdminShopParametersMetaControllerPostProcessUrlSchemaBefore');
        }

        return true;
    }

    /**
     * @return bool
     */
    private function copySymfonyServices()
    {
        $dir = __DIR__;
        if (version_compare(_PS_VERSION_, '8.0', '>=') && !@file_exists($dir . '/config/services.yml')) {
            if (function_exists('\symlink')) {
                if (!@symlink($dir . '/config/services__.yml', $dir . '/config/services.yml') || !@readlink($dir . '/config/services.yml')) {
                    Tools::copy($dir . '/config/services__.yml', $dir . '/config/services.yml');
                }
            } else {
                Tools::copy($dir . '/config/services__.yml', $dir . '/config/services.yml');
            }

            return @file_exists($dir . '/config/services.yml');
        }

        if (version_compare(_PS_VERSION_, '8.0', '<')) {
            @file_exists($dir . '/config/services.yml') && @unlink($dir . '/config/services.yml');

            return !@file_exists($dir . '/config/services.yml');
        }

        return true;
    }

    public function _installOverried()
    {
        $this->copy_directory(dirname(__FILE__) . '/views/templates/admin/_configure/templates', _PS_OVERRIDE_DIR_ . 'controllers/admin/templates');
        /* twig */
        @rename($this->getLocalPath() . '/views/PrestaShop___', $this->getLocalPath() . '/views/PrestaShop');

        return true;
    }

    /**
     * _unInstallOverried.
     *
     * @return bool
     */
    public function _uninstallOverried()
    {
        $this->delete_directory(_PS_OVERRIDE_DIR_ . 'controllers/admin/templates');
        /* twig */
        if (!$this->isExcludedHost(Tools::getHttpHost())) {
            @rename($this->getLocalPath() . '/views/PrestaShop', $this->getLocalPath() . '/views/PrestaShop___');
        }

        return true;
    }

    /**
     * Check if host is exclude
     *
     * @param string $host
     *
     * @return bool
     */
    private function isExcludedHost($host)
    {
        $rs = false;
        if (!class_exists('\EtsAwuStrHelper')) {
            require_once __DIR__ . '/classes/utils/EtsAwuStrHelper.php';
        }
        foreach ($this->_excludedHosts as $excludedHost) {
            if (EtsAwuStrHelper::contains($excludedHost, '*.')) {
                $excludedHost = explode('*.', $excludedHost)[1];
                if (EtsAwuStrHelper::endsWith($host, $excludedHost) !== false) {
                    $rs = true;
                }
            } elseif ($host === $excludedHost) {
                $rs = true;
            }
            if ($rs) {
                return true;
            }
        }

        return false;
    }

    /**
     * copy_directory.
     *
     * @param string $src
     * @param string $dst
     *
     * @return void
     */
    public function copy_directory($src, $dst)
    {
        if (is_dir($src)) {
            $dir = opendir($src);
            @mkdir($dst);
            while (false !== ($file = readdir($dir))) {
                if (('.' != $file) && ('..' != $file)) {
                    if (is_dir($src . '/' . $file)) {
                        $this->copy_directory($src . '/' . $file, $dst . '/' . $file);
                    } else {
                        if (file_exists($dst . '/' . $file) && 'index.php' != $file && ($content = Tools::file_get_contents($dst . '/' . $file)) && false === Tools::strpos($content, 'overried_by_hinh_ets')) {
                            copy($dst . '/' . $file, $dst . '/backup_' . $file);
                        }
                        copy($src . '/' . $file, $dst . '/' . $file);
                    }
                }
            }
            closedir($dir);
        }
    }

    /**
     * delete_directory.
     *
     * @param string $directory
     *
     * @return void
     */
    public function delete_directory($directory)
    {
        if (is_dir($directory)) {
            $dir = opendir($directory);
            while (false !== ($file = readdir($dir))) {
                if (('.' != $file) && ('..' != $file)) {
                    if (is_dir($directory . '/' . $file)) {
                        $this->delete_directory($directory . '/' . $file);
                    } else {
                        if (file_exists($directory . '/' . $file) && 'index.php' != $file && ($content = Tools::file_get_contents($directory . '/' . $file)) && false !== Tools::strpos($content, 'overried_by_hinh_ets')) {
                            @unlink($directory . '/' . $file);
                            if (file_exists($directory . '/backup_' . $file)) {
                                copy($directory . '/backup_' . $file, $directory . '/' . $file);
                            }
                        }
                    }
                }
            }
            closedir($dir);
        }
    }

    public function uninstall()
    {
        $etsDef = EtsAwuDefine::getInstance();

        return parent::uninstall()
            && $this->_uninstallTabs()
            && $etsDef->uninstallDb()
            && $this->_uninstallOverried()
            && $this->restoreSeoUrlConfig()
            && $this->removeSitemap()
            && EtsAwuSetting::deleteConfigKey();
    }

    private function safeMkDir($path, $permission = 0755)
    {
        if (!@mkdir($concurrentDirectory = $path, $permission) && !is_dir($concurrentDirectory)) {
            throw new \PrestaShopException(sprintf('Directory "%s" was not created', $concurrentDirectory));
        }

        return true;
    }

    private function checkOverrideDir()
    {
        if (defined('_PS_OVERRIDE_DIR_')) {
            $psOverride = @realpath(_PS_OVERRIDE_DIR_) . DIRECTORY_SEPARATOR;
            if (!is_dir($psOverride)) {
                $this->safeMkDir($psOverride);
            }
            $base = str_replace('/', DIRECTORY_SEPARATOR, $this->getLocalPath() . 'override');
            $iterator = new RecursiveDirectoryIterator($base, FilesystemIterator::SKIP_DOTS);
            /** @var RecursiveIteratorIterator|\SplFileInfo[] $iterator */
            $iterator = new RecursiveIteratorIterator($iterator, RecursiveIteratorIterator::SELF_FIRST);
            $iterator->setMaxDepth(4);
            foreach ($iterator as $k => $item) {
                if (!$item->isDir()) {
                    continue;
                }
                $path = str_replace($base . DIRECTORY_SEPARATOR, '', $item->getPathname());
                if (!@file_exists($psOverride . $path)) {
                    $this->safeMkDir($psOverride . $path);
                    @touch($psOverride . $path . DIRECTORY_SEPARATOR . '_do_not_remove');
                }
            }
            if (!file_exists($psOverride . 'index.php')) {
                Tools::copy($this->getLocalPath() . 'index.php', $psOverride . 'index.php');
            }
        }
    }

    public function disable($force_all = false)
    {
        if (parent::disable($force_all)) {
            if (!$force_all && EtsAwuDefine::checkEnableOtherShop($this->id)) {
                if (property_exists('Tab', 'enabled') && method_exists($this, 'get') && $dispatcher = $this->get('event_dispatcher')) {
                    /* @var \Symfony\Component\EventDispatcher\Debug\TraceableEventDispatcher|\Symfony\Component\EventDispatcher\EventDispatcherInterface $dispatcher */
                    $dispatcher->addListener(\PrestaShopBundle\Event\ModuleManagementEvent::DISABLE, function (PrestaShopBundle\Event\ModuleManagementEvent $event) {
                        EtsAwuDefine::activeTab($this->name);
                    });
                }
                if ($this->getOverrides() != null) {
                    try {
                        $this->installOverrides();
                    } catch (Exception $e) {
                        if ($e) {
                        }
                    }
                }
            } else {
                $this->restoreSeoUrlConfig()
                && $this->removeSitemap()
                && $this->_uninstallOverried();
            }

            return true;
        }

        return false;
    }

    public function enable($force_all = false)
    {
        if (self::isEnabled('ets_seo')) {
            throw new \PrestaShopException($this->l('ETS SEO Audit module is installed & enabled. You cannot enable this module'));
        }
        if (self::isEnabled('etsdynamicsitemap')) {
            throw new \PrestaShopException($this->l('ETS Dynamic Sitemap module is installed & enabled. You cannot enable this module'));
        }
        if (Module::isEnabled('prettyurls')) {
            throw new \PrestaShopException($this->l('Pretty Url module is installed & enabled. You cannot enable this module'));
        }
        if (!$force_all && EtsAwuDefine::checkEnableOtherShop($this->id) && $this->getOverrides() != null) {
            try {
                $this->uninstallOverrides();
            } catch (Exception $e) {
                if ($e) {
                }
            }
        }
        $this->checkOverrideDir();
        $parentResult = parent::enable($force_all)
            && $this->setSitemap()
            && $this->_installOverried()
            && $this->copySymfonyServices();

        return $parentResult;
    }

    private function _uninstallTabs()
    {
        $etsDef = EtsAwuDefine::getInstance();
        $tabs = $etsDef->adminControllers();
        foreach ($tabs as $key => $item) {
            if ($item) {
                $tabId = Tab::getIdFromClassName($key);
                if ((int) $tabId) {
                    $tab = new Tab((int) $tabId);
                    $tab->delete();
                }
            }
        }

        return true;
    }

    public function setRootSeoUrlConfig()
    {
        $etsDef = EtsAwuDefine::getInstance();
        $urlSchemaConfigs = $etsDef->seo_url_schema_configs();
        foreach ($urlSchemaConfigs as $k => $config) {
            Configuration::updateGlobalValue($config['root_name'], Configuration::get('PS_ROUTE_' . $k));
        }

        return true;
    }

    public function setDefaultConfig()
    {
        $etsDef = EtsAwuDefine::getInstance();
        $groups = $etsDef->getFieldConfig();
        $languages = Language::getLanguages(false);
        foreach ($groups as $configs) {
            foreach ($configs as $key => $config) {
                if (!Configuration::hasKey($key)) {
                    if (isset($config['default']) && '' !== $config['default']) {
                        Configuration::updateGlobalValue($key, $config['default']);
                    } else {
                        if (isset($config['type']) && ('textLang' == $config['type'] || 'textareaLang' == $config['type'] || 'selectLang' == $config['type'])) {
                            $value = [];
                            foreach ($languages as $lang) {
                                $value[$lang['id_lang']] = '';
                            }
                            Configuration::updateGlobalValue($key, $value);
                        } else {
                            Configuration::updateGlobalValue($key, '');
                        }
                    }
                }
            }
        }

        return true;
    }

    public function restoreSeoUrlConfig()
    {
        Configuration::updateValue('ETS_AWU_ENABLE_REMOVE_ID_IN_URL', 0);
        Configuration::updateValue('ETS_AWU_ENABLE_REMOVE_ATTR_ALIAS', 0);
        Configuration::updateValue('ETS_AWU_ENABLE_REMOVE_ID_ATTR_ALIAS', 0);
        Configuration::updateValue('ETS_AWU_ENABLE_REMOVE_LANG_CODE_IN_URL', 0);
        Configuration::updateValue('ETS_AWU_UPDATE_DUPLICATE_REWRITE', 0);
        Configuration::updateValue('ETS_AWU_SET_REMOVE_ID', 0);
        $etsDef = EtsAwuDefine::getInstance();
        foreach ($etsDef->seo_url_schema_configs() as $rule => $name) {
            foreach (Shop::getShops() as $shop) {
                if ($configRule = Configuration::get($name['root_name'], null, null, $shop['id_shop'])) {
                    if ('module' !== $rule && false !== strpos($configRule, '{id}')) {
                        Configuration::updateValue('PS_ROUTE_' . $rule, $configRule, false, null, $shop['id_shop']);
                    } else {
                        Configuration::updateValue('PS_ROUTE_' . $rule, $name['default'], false, null, $shop['id_shop']);
                    }
                } else {
                    Configuration::updateValue('PS_ROUTE_' . $rule, $name['default'], false, null, $shop['id_shop']);
                }
            }
        }

        return true;
    }

    public function processAfterSaveConfig()
    {
        Tools::clearCache();
        /* Update product has duplicate link_rewrite */
        $ETS_AWU_ENABLE_REMOVE_ID_IN_URL = (int) Tools::getValue('ETS_AWU_ENABLE_REMOVE_ID_IN_URL');
        if ((int) $ETS_AWU_ENABLE_REMOVE_ID_IN_URL) {
            if (!(int) Configuration::get('ETS_AWU_UPDATE_DUPLICATE_REWRITE')) {
                $seoUpdating = new EtsAwuUpdating();
                $seoUpdating->updateDuplicateProduct();
                $seoUpdating->updateDuplicateCategory();
                $seoUpdating->updateDuplicateCMS();
                $seoUpdating->updateDuplicateCMSCategory();
                $seoUpdating->updateDuplicateMeta();
                Configuration::updateValue('ETS_AWU_UPDATE_DUPLICATE_REWRITE', 1);
            }
        }

        /* Delete cache in others module to accept new configurations */
        if ((int) Configuration::get('ETS_SPEED_ENABLE_PAGE_CACHE') && Module::isInstalled('ets_superspeed') && Module::isEnabled('ets_superspeed') && class_exists('Ets_ss_class_cache')) {
            $cacheObjSuperSpeed = new Ets_ss_class_cache();
            if (method_exists($cacheObjSuperSpeed, 'deleteCache')) {
                $cacheObjSuperSpeed->deleteCache('index');
            }
        }
        if ((int) Configuration::get('ETS_SPEED_ENABLE_PAGE_CACHE') && Module::isInstalled('ets_pagecache') && Module::isEnabled('ets_pagecache') && class_exists('Ets_pagecache_class_cache')) {
            $cacheObjPageCache = new Ets_ss_class_cache();
            if (method_exists($cacheObjPageCache, 'deleteCache')) {
                $cacheObjPageCache->deleteCache('index');
            }
        }
    }

    public function hookDisplayBackOfficeHeader()
    {
        $controller = Tools::getValue('controller');
        $etsDef = EtsAwuDefine::getInstance();
        if ('AdminMeta' == $controller) {
            $linkRewriteRules = $etsDef->urlRules();
            $this->smarty->assign([
                'ETS_AWU_ENABLE_REMOVE_ID_IN_URL' => (int) Configuration::get('ETS_AWU_ENABLE_REMOVE_ID_IN_URL'),
                'ETS_AWU_ENABLE_REMOVE_LANG_CODE_IN_URL' => (int) Configuration::get('ETS_AWU_ENABLE_REMOVE_LANG_CODE_IN_URL'),
                'ETS_AWU_ENABLE_REMOVE_ATTR_ALIAS' => (int) Configuration::get('ETS_AWU_ENABLE_REMOVE_ATTR_ALIAS'),
                'ETS_AWU_ENABLE_REMOVE_ID_ATTR_ALIAS' => (int) Configuration::get('ETS_AWU_ENABLE_REMOVE_ID_ATTR_ALIAS'),
                'ETS_AWU_ENABLE_REDIRECT_NOTFOUND' => (int) Configuration::get('ETS_AWU_ENABLE_REDIRECT_NOTFOUND'),
                'ETS_AWU_REDIRECT_STATUS_CODE' => Configuration::get('ETS_AWU_REDIRECT_STATUS_CODE'),
                'linkRewriteRules' => $linkRewriteRules,
            ]);
            if (version_compare(_PS_VERSION_, '8.0.0', '<')) {
                if ($this->getRequestContainer()) {
                    $this->addTwigVar('etsAwuRemoveIdField', $this->display(__FILE__, 'remove_id_field.tpl'));
                    $this->addTwigVar('etsAwuRedirectUrl', $this->display(__FILE__, 'redirect_url_field.tpl'));
                }
            }
        }
        if (version_compare(_PS_VERSION_, '8.0.0', '<')) {
            if ($this->getRequestContainer()) {
                $this->addTwigVar('ETS_AWU_IS_178', $this->is178);
            }
        }
        $languages = Language::getLanguages(false);
        $ets_languages = [];
        foreach ($languages as $lang) {
            $ets_languages[$lang['iso_code']] = $lang['id_lang'];
        }
        $this->context->controller->addCSS($this->_path . 'views/css/admin.css');
        $this->smarty->assign([
            'ets_awu_defined' => [
                'is176' => (int) $this->is176,
                'is175' => (int) $this->is175,
                'isSf' => $this->getRequestContainer(),
                'seo_analysis_rules' => '',
                'readability_rules' => '',
                'transition_words' => '',
                'id_current_page' => '',
                'meta_tamplate_configured' => '',
                'placeholder_meta' => '',
            ],
            'is16' => !$this->is17,
            'linkAdminJs' => $this->_path . 'views/js/admin.js',
            'link_analysis_js' => $this->_path . 'views/js/analysis.js',
            'link_select2_js' => $this->_path . 'views/js/select2.min.js',
            'isSf' => $this->getRequestContainer() ? 1 : 0,
            'current_lang_selected' => 'AdminProducts' == $controller ? $languages[0]['id_lang'] : Configuration::get('PS_LANG_DEFAULT'),
            'controller' => $controller,
            'ets_languages' => $ets_languages,
        ]);
        if (version_compare(_PS_VERSION_, '8.0.0', '<')) {
            if ('AdminCmsContent' == $controller) {
                if ($this->isCmsCategoryPage()) {
                    $this->seo_cms_category_html();
                } else {
                    $this->seo_cms_html();
                }
            }
            if ($this->isMetaController()) {
                $this->seo_meta_html();
            } elseif ('AdminCategories' == $controller) {
                $this->seo_category_html();
            } elseif ('AdminManufacturers' == $controller) {
                $this->seo_manufacturer_html();
            } elseif ('AdminSuppliers' == $controller) {
                $this->seo_supplier_html();
            }
        }
        if ($errorLinkRewrite = $this->context->cookie->__get('ets_awu_error_link_rewrite')) {
            $this->context->controller->errors = [$errorLinkRewrite];
            $this->context->cookie->__unset('ets_awu_error_link_rewrite');
        }
        Media::addJsDef(['etsAwuBo' => ['adminControllerOverrides' => EtsAwuDefine::getInstance()->getListAdminControllerOverrides(), 'currentController' => $controller],
        ]);

        return $this->display(__FILE__, 'admin_head.tpl');
    }

    public function hookActionObjectUpdateBefore($params)
    {
        if (!defined('_PS_ADMIN_DIR_')) {
            return;
        }
        if (!Tools::getIsset('PS_ROUTE_category_rule') && !$this->is178) {
            $this->updateConfigSeoNoid();
        }
        if (!defined('_PS_ADMIN_DIR_')) {
            return;
        }
        if ($this->getRequestContainer()) {
            if (isset($params['object']) && $params['object'] instanceof Product && preg_match('/sell\/catalog\/products/', $_SERVER['REQUEST_URI'])) {
                $error = null;
                $seoAdvanced = ($seoAdvanced = Tools::getValue('ets_awu_advanced')) && is_array($seoAdvanced) ? $seoAdvanced : [];
                if (isset($seoAdvanced['canonical_url']) && !$error) {
                    foreach ($seoAdvanced['canonical_url'] as $id_lang => $url) {
                        if ($url && !Validate::isAbsoluteUrl($url)) {
                            $error = sprintf($this->l('[%s] The Canonical url must start with http:// or https:// '), Language::getIsoById($id_lang));
                            break;
                        }
                    }
                }
                if ($error) {
                    throw new PrestaShopException($error);
                }
            }
        } else {
            $this->validateLinkRewrite($params);
        }
    }

    public function hookActionAdminMetaControllerUpdate_optionsBefore()
    {
        if (Tools::getIsset('PS_ROUTE_category_rule') && !$this->is178) {
            $this->updateConfigSeoNoid();
        }
    }

    public function hookActionAdminMetaControllerUpdate_optionsAfter()
    {
        if (Tools::getIsset('PS_ROUTE_category_rule') && Tools::getIsset('ETS_AWU_ENABLE_REMOVE_ID_IN_URL')) {
            $etsDef = EtsAwuDefine::getInstance();
            $urlSchemaConfigs = $etsDef->seo_url_schema_configs();
            foreach ($urlSchemaConfigs as $k => $config) {
                if ($config) {
                    $val = Tools::getValue('PS_ROUTE_' . $k);
                    if (Validate::isCleanHtml($val)) {
                        Configuration::updateValue('PS_ROUTE_' . $k, $val);
                    }
                }
            }
        }
    }

    private function transTwig()
    {
        return [
            'Content' => $this->l('Content', 'twig'),
            'SEO settings' => $this->l('SEO settings', 'twig'),
            'Search Engine Optimization' => $this->l('Search Engine Optimization', 'twig'),
            'Improve your ranking and how your product page will appear in search engines results.' => $this->l('Improve your ranking and how your product page will appear in search engines results.', 'twig'),
            'You now have three default customer groups.' => $this->l('You now have three default customer groups.', 'twig'),
            '%s - All people without a valid customer account.' => $this->l('%s - All people without a valid customer account.', 'twig'),
            '%s - Customer who placed an order with the guest checkout.' => $this->l('%s - Customer who placed an order with the guest checkout.', 'twig'),
            '%s - All people who have created an account on this site.' => $this->l('%s - All people who have created an account on this site.', 'twig'),
            'Shop association' => $this->l('Shop association', 'twig'),
            'To add tags, click in the field, write something, and then press the "Enter" key.' => $this->l('To add tags, click in the field, write something, and then press the "Enter" key.', 'twig'),
        ];
    }

    public function getTwigs()
    {
        $controller = Tools::getValue('controller');
        $twigs = [
            'AWU_TWIG_TRANS' => $this->transTwig(),
        ];
        $this->context->smarty->assign(
            [
                'is16' => $this->is16,
            ]
        );
        if ($this->isMetaController()) {
            $this->assignPageParams('meta');
            $linkRewriteRules = EtsAwuDefine::getInstance()->urlRules();
            $this->smarty->assign([
                'ETS_AWU_ENABLE_REMOVE_ID_IN_URL' => (int) Configuration::get('ETS_AWU_ENABLE_REMOVE_ID_IN_URL'),
                'ETS_AWU_ENABLE_REMOVE_LANG_CODE_IN_URL' => (int) Configuration::get('ETS_AWU_ENABLE_REMOVE_LANG_CODE_IN_URL'),
                'ETS_AWU_ENABLE_REMOVE_ATTR_ALIAS' => (int) Configuration::get('ETS_AWU_ENABLE_REMOVE_ATTR_ALIAS'),
                'ETS_AWU_ENABLE_REMOVE_ID_ATTR_ALIAS' => (int) Configuration::get('ETS_AWU_ENABLE_REMOVE_ID_ATTR_ALIAS'),
                'ETS_AWU_ENABLE_REDIRECT_NOTFOUND' => (int) Configuration::get('ETS_AWU_ENABLE_REDIRECT_NOTFOUND'),
                'ETS_AWU_REDIRECT_STATUS_CODE' => Configuration::get('ETS_AWU_REDIRECT_STATUS_CODE'),
                'linkRewriteRules' => $linkRewriteRules,
            ]);
            $twigs['etsAwuRemoveIdField'] = $this->display(__FILE__, 'remove_id_field.tpl');
            $twigs['etsAwuRedirectUrl'] = $this->display(__FILE__, 'redirect_url_field.tpl');
            $twigs['ets_meta_seo_setting_html'] = $this->display(__FILE__, 'page/seo_setting.tpl');
            $twigs['ETS_AWU_ENABLE_REMOVE_ID_IN_URL'] = (int) Configuration::get('ETS_AWU_ENABLE_REMOVE_ID_IN_URL');
            $twigs['ETS_AWU_ENABLE_REMOVE_ATTR_ALIAS'] = (int) Configuration::get('ETS_AWU_ENABLE_REMOVE_ATTR_ALIAS');
            $twigs['ETS_AWU_ENABLE_REMOVE_ID_ATTR_ALIAS'] = (bool) Configuration::get('ETS_AWU_ENABLE_REMOVE_ID_ATTR_ALIAS');
            $twigs['ETS_AWU_ENABLE_REDRECT_NOTFOUND'] = (int) Configuration::get('ETS_AWU_ENABLE_REDRECT_NOTFOUND');
            $twigs['ETS_AWU_REDIRECT_STATUS_CODE'] = (int) Configuration::get('ETS_AWU_REDIRECT_STATUS_CODE');
            $twigs['ETS_AWU_ENABLE_REMOVE_LANG_CODE_IN_URL'] = (int) Configuration::get('ETS_AWU_ENABLE_REMOVE_LANG_CODE_IN_URL');
            $twigs['titleRemoveLangCode'] = $this->l('Remove ISO code in URL for default language');
            $twigs['titleRemoveAttrAlias'] = $this->l('Remove attribute alias in URL');
        }
        if ('AdminCategories' == $controller) {
            $this->assignPageParams('category');
            $twigs['ets_category_seo_setting_html'] = $this->display(__FILE__, 'page/seo_setting.tpl');
        }
        if ('AdminManufacturers' == $controller) {
            $this->assignPageParams('manufacturer');
            $twigs['ets_manufacturer_seo_setting_html'] = $this->display(__FILE__, 'page/seo_setting.tpl');
        }
        if ('AdminSuppliers' == $controller) {
            $this->assignPageParams('supplier');
            $twigs['ets_supplier_seo_setting_html'] = $this->display(__FILE__, 'page/seo_setting.tpl');
        }
        if ('AdminCmsContent' == $controller) {
            if ($this->isCmsCategoryPage()) {
                $this->assignPageParams('cms_category');
                $twigs['ets_cms_category_seo_setting_html'] = $this->display(__FILE__, 'page/seo_setting.tpl');
            } else {
                $this->assignPageParams('cms');
                $twigs['ets_cms_seo_setting_html'] = $this->display(__FILE__, 'page/seo_setting.tpl');
            }
        }
        $twigs['ETS_AWU_IS_178'] = $this->is178;
        if ('AdminProducts' == $controller || (self::getRequestContainer() && in_array(self::getRequestContainer()->get('_route'), ['admin_product_form', 'admin_product_catalog']))) {
            if (!Tools::isSubmit('ajax')) {
                if ($request = $this->getRequestContainer()) {
                    $routes = [
                        'admin_product_form' => 'id',
                        'admin_products_edit' => 'productId',
                    ];
                    if (array_key_exists($key = $request->get('_route'), $routes)) {
                        $idProduct = (int) $request->get($routes[$key]);
                    }
                }
                if (isset($idProduct) && $idProduct) {
                    $languages = Language::getLanguages(true);
                    $current_lang = [
                        'id' => $this->context->language->id,
                        'iso_code' => $this->context->language->iso_code,
                    ];
                    $awuDef = EtsAwuDefine::getInstance();
                    $this->smarty->assign([
                        'ets_awu_languages' => $languages,
                        'languages' => $languages,
                        'current_lang' => $current_lang,
                        'seo_advanced' => $awuDef->seo_advanced('product', $idProduct, $this->context),
                        'is_new_theme' => $this->getRequestContainer() ? true : false,
                        'seo_enabled' => 1,
                        'is16' => $this->is16,
                        'newForm' => true,
                    ]);
                    $twigs['ets_seo_advanced'] = $this->display(__FILE__, 'parts/_seo_advanced.tpl');
                }
            }
        }

        return $twigs;
    }

    public function assignTwigVar($params)
    {
        /* @var \Twig\Environment $tw */
        if (!class_exists('Ets_awesomeurl_twig')) {
            require_once dirname(__FILE__) . '/classes/Ets_awesomeurl_twig.php';
        }
        if ($sfContainer = self::getSfContainer()) {
            try {
                $tw = $sfContainer->get('twig');
                $firstKey = array_keys($params)[0];
                if (!array_key_exists($firstKey, $tw->getGlobals())) {
                    $tw->addExtension(new Ets_awesomeurl_twig($params));
                }
            } catch (\Twig\Error\RuntimeError $e) {
                // do no thing
            }
        }
    }

    public function addTwigVar($key, $value)
    {
        if ($sfContainer = self::getSfContainer()) {
            $sfContainer->get('twig')->addGlobal($key, $value);
        }
    }

    public function hookActionDispatcherBefore($params)
    {
        $context = $this->context;

        if (isset($params['controller_type']) && Dispatcher::FC_ADMIN == $params['controller_type'] && isset($context->employee->id) && $context->employee->id && $context->cookie->passwd && $context->employee->isLoggedBack()) {
            if (version_compare(_PS_VERSION_, '8.0.0', '>=')) {
                if ($twigs = $this->getTwigs()) {
                    $this->assignTwigVar(
                        $twigs
                    );
                }
            }
        }
    }

    public function hookActionAdminEtsAwuUrlRedirectsFormModifier($params)
    {
        if (isset($this->context->cookie->ets_awu_redirect_values)) {
            $params['fields_value'] = array_merge($params['fields_value'], json_decode($this->context->cookie->__get('ets_awu_redirect_values'), true));
            $this->context->cookie->__unset('ets_awu_redirect_values');
        }
    }

    public function hookDisplayHeader()
    {
        $currentCtl = Tools::getValue('controller');
        $isModuleCtl = Tools::getValue('fc') && 'module' === Tools::getValue('fc');
        if ($isModuleCtl && 'preview' === $currentCtl && 'creativeelements' === Tools::getValue('module')) {
            return '';
        }
        $this->context->controller->addCss([
            $this->_path . 'views/css/front.css',
        ], 'all');
        $this->context->controller->addJS($this->_path . 'views/js/helpers.js');
        $etsAwuFo = [
            'version' => $this->version,
            'conf' => [
                'removeId' => (bool) Configuration::get('ETS_AWU_ENABLE_REMOVE_ID_IN_URL'),
                'removeAttrAlias' => (bool) Configuration::get('ETS_AWU_ENABLE_REMOVE_ATTR_ALIAS'),
                'removeAttrIdAlias' => (bool) Configuration::get('ETS_AWU_ENABLE_REMOVE_ID_ATTR_ALIAS'),
            ],
        ];
        if ('product' === $currentCtl && ($prodGroups = $this->context->smarty->getTemplateVars('groups'))) {
            $etsAwuFo['productHasGroups'] = true;
            $anchorSeparator = Configuration::get('PS_ATTRIBUTE_ANCHOR_SEPARATOR');
            $etsAwuFo['anchorSeparator'] = $anchorSeparator;
            $groups = [];
            $_getAttrUrl = static function ($v) use ($anchorSeparator) {
                return str_replace($anchorSeparator, '_', Tools::link_rewrite($v));
            };
            foreach ($prodGroups as $idGroup => $prodGroup) {
                $name = $_getAttrUrl($prodGroup['group_name']);
                $curGroup = ['idGroup' => $idGroup, 'type' => 'select' === $prodGroup['group_type'] ? 'select' : 'input', 'attributes' => []];
                foreach ($prodGroup['attributes'] as $idAttr => $attribute) {
                    $curGroup['attributes'][] = [
                        'id' => $idAttr,
                        'idAttribute' => $idAttr,
                        'name' => $attribute['name'],
                        'url' => $_getAttrUrl($attribute['name']),
                        'idGroup' => $idGroup,
                    ];
                }
                $groups[$name] = $curGroup;
            }
            $etsAwuFo['productGroups'] = $groups;
            $etsAwuFo['productId'] = Tools::getValue('id_product');
            $this->context->controller->addJS($this->_path . '/views/js/front.js');
        }
        Media::addJsDef(['etsAwuFo' => $etsAwuFo]);

        $this->getSeoMetaData();
        if (!$this->is17) {
            return $this->display(__FILE__, 'head.tpl');
        }
    }

    public function hookActionObjectUpdateAfter($params)
    {
        $seoSetting = EtsAwuSetting::getInstance();
        $seoSetting->updateSeoCms($params);
        $seoSetting->updateSeoMeta($params);
        $seoSetting->updateSeoCategory($params);
        $seoSetting->updateSeoCmsCategory($params);
        $seoSetting->updateSeoManufacturer($params);
        $seoSetting->updateSeoSupplier($params);
        $ETS_AWU_ENABLE_REMOVE_ID_IN_URL = (int) Tools::getValue('ETS_AWU_ENABLE_REMOVE_ID_IN_URL');
        if ((int) $ETS_AWU_ENABLE_REMOVE_ID_IN_URL) {
            $this->processAfterSaveConfig();
        }
    }

    /**
     * hookActionObjectAddAfter.
     *
     * @param array $params
     *
     * @return void
     */
    public function hookActionObjectAddAfter($params)
    {
        $seoSetting = EtsAwuSetting::getInstance();
        $seoSetting->updateSeoCms($params);
        $seoSetting->updateSeoMeta($params);
        $seoSetting->updateSeoCategory($params);
        $seoSetting->updateSeoCmsCategory($params);
        $seoSetting->updateSeoManufacturer($params);
        $seoSetting->updateSeoSupplier($params);
    }

    public function updateConfigSeoNoid()
    {
        if (Tools::getIsset('ETS_AWU_ENABLE_REMOVE_ID_IN_URL')) {
            $ETS_AWU_ENABLE_REMOVE_ID_IN_URL = (int) Tools::getValue('ETS_AWU_ENABLE_REMOVE_ID_IN_URL');
            $params = null;
            if ($this->getRequestContainer()) {
                if ($metaData = Tools::getValue('meta_settings_form')) {
                    $params = $metaData;
                } elseif ($metaData = Tools::getValue('meta_settings_url_schema_form')) {
                    $params = [];
                    $params['url_schema'] = $metaData;
                }
            }

            $etsDef = EtsAwuDefine::getInstance();
            $urlSchemaConfigs = $etsDef->seo_url_schema_configs();
            /* UPDATE schema configs */

            if (!(int) Configuration::get('ETS_AWU_ENABLE_REMOVE_ID_IN_URL') && (int) $ETS_AWU_ENABLE_REMOVE_ID_IN_URL) {
                if (!(int) Configuration::get('ETS_AWU_SET_REMOVE_ID')) {
                    Configuration::updateValue('ETS_AWU_SET_REMOVE_ID', 1);
                }
                foreach ($urlSchemaConfigs as $k => $config) {
                    if ($params && isset($params['url_schema'][$k])) {
                        $dataConfig = $params['url_schema'][$k];
                    } else {
                        $dataConfig = Tools::getValue('PS_ROUTE_' . $k);
                    }
                    if ($dataConfig && Validate::isCleanHtml($dataConfig)) {
                        $prevConfig = Configuration::get('PS_ROUTE_' . $k);
                        if (!$prevConfig && isset($config['default']) && $config['default']) {
                            $prevConfig = $config['default'];
                        }
                        Configuration::updateValue($config['no_id'], $dataConfig);
                        Configuration::updateValue($config['name'], $prevConfig);
                        $oldConfig = Configuration::get($config['old_name']);
                        $rootConfig = Configuration::get($config['root_name']);
                        if (!$oldConfig || ($oldConfig && 'module' != $k && !preg_match('/\{id\}/', $oldConfig))) {
                            Configuration::updateValue($config['old_name'], $prevConfig);
                        }
                        if (!$rootConfig || ($rootConfig && 'module' != $k && !preg_match('/\{id\}/', $rootConfig))) {
                            Configuration::updateValue($config['root_name'], $prevConfig);
                        }
                    }
                }
            }

            $ETS_AWU_ENABLE_REMOVE_LANG_CODE_IN_URL = (int) Tools::getValue('ETS_AWU_ENABLE_REMOVE_LANG_CODE_IN_URL');
            $ETS_AWU_ENABLE_REMOVE_ATTR_ALIAS = (int) Tools::getValue('ETS_AWU_ENABLE_REMOVE_ATTR_ALIAS');
            $ETS_AWU_ENABLE_REMOVE_ID_ATTR_ALIAS = (int) Tools::getValue('ETS_AWU_ENABLE_REMOVE_ID_ATTR_ALIAS');
            $ETS_AWU_ENABLE_REDIRECT_NOTFOUND = (int) Tools::getValue('ETS_AWU_ENABLE_REDIRECT_NOTFOUND');
            Configuration::updateValue('ETS_AWU_ENABLE_REMOVE_ID_IN_URL', (int) $ETS_AWU_ENABLE_REMOVE_ID_IN_URL);
            Configuration::updateValue('ETS_AWU_ENABLE_REMOVE_LANG_CODE_IN_URL', (int) $ETS_AWU_ENABLE_REMOVE_LANG_CODE_IN_URL);
            Configuration::updateValue('ETS_AWU_ENABLE_REMOVE_ATTR_ALIAS', (int) $ETS_AWU_ENABLE_REMOVE_ATTR_ALIAS);
            Configuration::updateValue('ETS_AWU_ENABLE_REMOVE_ID_ATTR_ALIAS', (int) $ETS_AWU_ENABLE_REMOVE_ID_ATTR_ALIAS);
            Configuration::updateValue('ETS_AWU_ENABLE_REDIRECT_NOTFOUND', (int) $ETS_AWU_ENABLE_REDIRECT_NOTFOUND);
            $ETS_AWU_REDIRECT_STATUS_CODE = (int) Tools::getValue('ETS_AWU_REDIRECT_STATUS_CODE');
            if ($ETS_AWU_REDIRECT_STATUS_CODE) {
                Configuration::updateValue('ETS_AWU_REDIRECT_STATUS_CODE', (int) $ETS_AWU_REDIRECT_STATUS_CODE);
            }
        }
    }

    public function getContent()
    {
        try {
            $urlRedirect = $this->context->link->getAdminLink('AdminMeta', true, ['route' => 'admin_metas_index']);
        } catch (Exception $ex) {
            $urlRedirect = $this->context->link->getAdminLink('AdminMeta', true);
        }

        Tools::redirectAdmin($urlRedirect);
    }

    public function hookActionAdminShopParametersMetaControllerPostProcessUrlSchemaBefore($params)
    {
        if ($this->is178) {
            $this->updateConfigSeoNoid();
        }
    }

    public function getLinkDuplicate($type, $params)
    {
        $duplicate_link = '';
        $duplicate_title = '';
        if ('product' == $type) {
            $sfP = ['route' => 'admin_product_form', 'id' => $params['id']];
            $p = ['id_product' => $params['id'], 'updateproduct' => true];
            $duplicate_link = $this->getPageLink('AdminProducts', $sfP, $p);
            $duplicate_title = $params['title'];
        } elseif ('category' == $type) {
            $sfP = ['route' => 'admin_categories_edit', 'categoryId' => $params['id']];
            $p = ['id_category' => $params['id'], 'updatecategory' => true];
            $duplicate_link = $this->getPageLink('AdminCategories', $sfP, $p);
            $duplicate_title = $params['title'];
        } elseif ('cms' == $type) {
            $sfP = ['route' => 'admin_cms_pages_edit', 'cmsPageId' => $params['id']];
            $p = ['id_cms' => $params['id'], 'updatecms' => true];
            $duplicate_link = $this->getPageLink('AdminCmsContent', $sfP, $p);
            $duplicate_title = $params['title'];
        } elseif ('cms_category' == $type) {
            $sfP = ['route' => 'admin_cms_pages_category_edit', 'cmsCategoryId' => $params['id']];
            $p = ['id_cms_category' => $params['id'], 'updatecms_category' => true];
            $duplicate_link = $this->getPageLink('AdminCmsContent', $sfP, $p);
            $duplicate_title = $params['title'];
        } elseif ('meta' == $type) {
            $sfP = ['route' => 'admin_metas_edit', 'metaId' => $params['id']];
            $p = ['id_meta' => $params['id'], 'updatemeta' => true];
            $duplicate_link = $this->getPageLink('AdminMeta', $sfP, $p);
            $duplicate_title = $params['title'];
        }
        if (!$this->is17 && isset($p)) {
            $duplicate_link .= '&' . http_build_query($p);
        }
        $this->smarty->assign([
            'duplicate_link' => $duplicate_link,
            'duplicate_title' => $duplicate_title,
        ]);

        return $this->display(__FILE__, 'duplicate_link.tpl');
    }

    public function getPageLink($controller, $sfParams, $params)
    {
        if ($this->is17) {
            try {
                return $this->context->link->getAdminLink($controller, true, $sfParams, $params);
            } catch (Exception $ex) {
                if ($ex) {
                }

                return $this->context->link->getAdminLink($controller, true, [], $params);
            }
        } else {
            return $this->context->link->getAdminLink($controller) . ($params ? '&' . http_build_query($params) : '');
        }

        return null;
    }

    public function getLinkUrlRedirect($item, $type = 'target')
    {
        if ('url' == $type) {
            $this->smarty->assign([
                'ets_target' => $item['url'],
                'ets_link' => $item['url'],
            ]);
        } else {
            $this->smarty->assign([
                'ets_target' => $item['target'],
                'ets_link' => false === strpos($item['target'], 'https://') && false === strpos($item['target'], 'http://') ? 'http://' . $item['target'] : $item['target'],
            ]);
        }

        return $this->display(__FILE__, 'url_redirect_link.tpl');
    }

    public function getPsExtraOption()
    {
        $etsDef = EtsAwuDefine::getInstance();

        return $etsDef->getFieldConfig()['ps_extra'];
    }

    public function setSitemap()
    {
        if ((int) Configuration::get('ETS_AWU_ENABLE_XML_SITEMAP') && (int) Configuration::get('PS_MULTISHOP_FEATURE_ACTIVE')) {
            if (@file_exists(_PS_ROOT_DIR_ . '/robots.txt')) {
                @rename(_PS_ROOT_DIR_ . '/robots.txt', _PS_ROOT_DIR_ . '/_robots.txt');
            }
            $path = _PS_ROOT_DIR_ . '/_robots.txt';
        } else {
            $path = _PS_ROOT_DIR_ . '/robots.txt';
        }
        if (@file_exists($path) && @is_writable($path)) {
            $robots = trim(Tools::file_get_contents($path));
        } else {
            $robots = '';
        }
        $robots = str_replace("\r\n", "\n", $robots);
        $robots = preg_replace('/^(Sitemap: .+index_sitemap.xml)$/im', '#$1', $robots);
        if ($shops = Shop::getShops(false)) {
            foreach ($shops as $shop) {
                $s = new Shop($shop['id_shop']);
                $shopUrl = $s->getBaseURL(true, true);
                if (!preg_match('/^Sitemap: ' . str_replace('/', "\/", $shopUrl) . 'sitemap.xml$/im', $robots)) {
                    $robots .= "\nSitemap: " . $shopUrl . 'sitemap.xml';
                }
            }
        }
        @file_put_contents($path, $robots);

        return true;
    }

    public function removeSitemap()
    {
        if (@file_exists(_PS_ROOT_DIR_ . '/_robots.txt')) {
            @rename(_PS_ROOT_DIR_ . '/_robots.txt', _PS_ROOT_DIR_ . '/robots.txt');
        }
        $path = _PS_ROOT_DIR_ . '/robots.txt';
        if (@file_exists($path) && @is_writable($path) && ($robots = Tools::file_get_contents($path))) {
            $robots = str_replace("\r\n", "\n", $robots);
            $robots = preg_replace('/^Sitemap: .+\/sitemap.xml$/im', '', $robots);
            $robots = preg_replace('/^#(Sitemap: .+index_sitemap.xml)$/im', '$1', $robots);
            @file_put_contents($path, $robots);
        }

        return true;
    }

    /**
     * hookDisplayAdminProductsSeoStepBottom.
     *
     * @param mixed $params
     *
     * @return void
     */
    public function hookDisplayAdminProductsSeoStepBottom($params)
    {
        $id_product = isset($params['id_product']) ? (int) $params['id_product'] : '';
        $languages = Language::getLanguages(true);
        $current_lang = [
            'id' => $this->context->language->id,
            'iso_code' => $this->context->language->iso_code,
        ];

        $awuDef = EtsAwuDefine::getInstance();

        $this->smarty->assign([
            'ets_awu_languages' => $languages,
            'languages' => $languages,
            'current_lang' => $current_lang,
            'seo_advanced' => $awuDef->seo_advanced('product', $id_product, $this->context),
            'is_new_theme' => $this->getRequestContainer() ? true : false,
            'seo_enabled' => 1,
            'is16' => $this->is16,
        ]);

        return $this->display(__FILE__, 'page/seo_setting.tpl');
    }

    public function hookActionProductSave($params)
    {
        EtsAwuSetting::getInstance()->updateSeoProduct($params);
    }

    public function assignPageParams($type)
    {
        $metaTitleConfigName = '';
        $metaDescConfigName = '';
        switch ($type) {
            case 'cms':
                $idKey = 'id_cms';
                $idKey176 = 'cmsPageId';
                $objPage = 'CMS';
                $metaTitleConfigName = 'ETS_AWU_CMS_META_TITLE';
                $metaDescConfigName = 'ETS_AWU_CMS_META_DESC';
                break;

            case 'cms_category':
                $idKey = 'id_cms_category';
                $idKey176 = 'cmsCategoryId';
                $objPage = 'CMSCategory';
                $metaTitleConfigName = 'ETS_AWU_CMS_CATE_META_TITLE';
                $metaDescConfigName = 'ETS_AWU_CMS_CATE_META_DESC';
                break;

            case 'meta':
                $idKey = 'id_meta';
                $idKey176 = 'metaId';
                $objPage = 'Meta';
                break;

            case 'category':
                $idKey = 'id_category';
                $idKey176 = 'categoryId';
                $objPage = 'Category';
                $metaTitleConfigName = 'ETS_AWU_CATEGORY_META_TITLE';
                $metaDescConfigName = 'ETS_AWU_CATEGORY_META_DESC';
                break;

            case 'manufacturer':
                $idKey = 'id_manufacturer';
                $idKey176 = 'manufacturerId';
                $objPage = 'Manufacturer';
                $metaTitleConfigName = 'ETS_AWU_MANUFACTURER_META_TITLE';
                $metaDescConfigName = 'ETS_AWU_MANUFACTURER_META_DESC';
                break;

            case 'supplier':
                $idKey = 'id_supplier';
                $idKey176 = 'supplierId';
                $objPage = 'Supplier';
                $metaTitleConfigName = 'ETS_AWU_SUPPLIER_META_TITLE';
                $metaDescConfigName = 'ETS_AWU_SUPPLIER_META_DESC';
                break;
        }

        $languages = Language::getLanguages(false);
        $metaConfig = [];
        foreach ($languages as $lang) {
            $metaConfig[$lang['id_lang']] = [
                'title' => $metaTitleConfigName ? (string) Configuration::get($metaTitleConfigName, $lang['id_lang']) : '',
                'desc' => $metaDescConfigName ? (string) Configuration::get($metaDescConfigName, $lang['id_lang']) : '',
            ];
        }

        $current_lang = [
            'id' => $this->context->language->id,
            'iso_code' => $this->context->language->iso_code,
        ];
        if (count($languages)) {
            $langDefault = Language::getLanguage((int) Configuration::get('PS_LANG_DEFAULT'));
            $current_lang = [
                'id' => $langDefault ? $langDefault['id_lang'] : $languages[0]['id_lang'],
                'iso_code' => $langDefault ? $langDefault['iso_code'] : $languages[0]['iso_code'],
            ];
        }
        $seoDef = EtsAwuDefine::getInstance();

        $seo_cms = [
            'link' => [],
            'link_rewrite' => [],
            'meta_description' => [],
            'meta_title' => [],
            'key_phrase' => [],
        ];
        $id = (int) Tools::getValue($idKey, null);
        if ($request = $this->getRequestContainer()) {
            $id = (int) $request->get($idKey176);
        }

        if ($id) {
            foreach ($languages as $lang) {
                $page = new $objPage($id, $lang['id_lang']);
                $link = '';
                if ('meta' == $type) {
                    $link = $page->url_rewrite ? $this->getPageLink($page->page, null, null) : '';
                } elseif ('cms' == $type) {
                    $link = $this->context->link->getCMSLink($page, null, null, (int) $lang['id_lang']);
                } elseif ('cms_category' == $type) {
                    $link = $this->context->link->getCMSCategoryLink($page, null, null, (int) $lang['id_lang']);
                } elseif ('category' == $type) {
                    $link = $this->context->link->getCategoryLink($page, $page->link_rewrite, $lang['id_lang']);
                } elseif ('manufacturer' == $type) {
                    $link = $this->context->link->getManufacturerLink($page, null, (int) $lang['id_lang'], $this->context->shop->id);
                } elseif ('supplier' == $type) {
                    $link = $this->context->link->getSupplierLink($page, null, (int) $lang['id_lang'], $this->context->shop->id);
                }
                $seo_cms['link'][$lang['id_lang']] = $link ? (is_array($link) ? $link[0]['link'] : $link) : '';
                $seo_cms['link_rewrite'][$lang['id_lang']] = isset($page->link_rewrite) ? $page->link_rewrite : (isset($page->url_rewrite) ? $page->url_rewrite : '');
                $seo_cms['meta_title'][$lang['id_lang']] = isset($page->meta_title) && $page->meta_title ? $page->meta_title : (isset($page->title) ? $page->title : '');
                $seo_cms['meta_description'][$lang['id_lang']] = isset($page->meta_description) && $page->meta_description ? $page->meta_description : (isset($page->description) ? $page->description : '');
                if (!$seo_cms['meta_title'][$lang['id_lang']] && isset($page->name) && $page->name) {
                    $seo_cms['meta_title'][$lang['id_lang']] = $page->name;
                }
                if ('manufacturer' == $type && !$page->meta_description) {
                    $seo_cms['meta_description'][$lang['id_lang']] = $page->short_description;
                }
            }
        }
        $this->smarty->assign([
            'ets_awu_languages' => $languages,
            'tmp_dir' => dirname(__FILE__) . '/views/templates',
            'seo_data' => '',
            'current_lang' => $current_lang,
            'seo_cms' => '',
            'seo_advanced' => $seoDef->seo_advanced($type, $id, $this->context),
            'analysis_types' => '',
            'seo_enabled' => 1,
            'enable_force_rating' => false,
            'readability_enabled' => 1,
            'is_new_theme' => $this->getRequestContainer() ? true : false,
            'enable_rating' => false,
            'rating_config' => '',
            'rating_setting' => '',
            'meta_config' => $metaConfig,
        ]);
    }

    public function seo_category_html()
    {
        $this->assignPageParams('category');
        $enableSeo = 1;
        if ($this->getRequestContainer()) {
            $controller = Tools::getValue('controller');
            if ('AdminCategories' == $controller) {
                $this->addTwigVar('ets_category_seo_setting_html', $enableSeo ? $this->display(__FILE__, 'page/seo_setting.tpl') : '');
            }
        } else {
            $this->context->smarty->assign([
                'ets_category_seo_setting_html' => $enableSeo ? $this->display(__FILE__, 'page/seo_setting.tpl') : '',
            ]);
        }
    }

    public function seo_manufacturer_html()
    {
        $this->assignPageParams('manufacturer');
        $enableSeo = 1;
        if ($this->getRequestContainer()) {
            $controller = Tools::getValue('controller');
            if ('AdminManufacturers' == $controller) {
                $this->addTwigVar('ets_manufacturer_seo_setting_html', $enableSeo ? $this->display(__FILE__, 'page/seo_setting.tpl') : '');
            }
        } else {
            $this->context->smarty->assign([
                'ets_manufacturer_seo_setting_html' => $enableSeo ? $this->display(__FILE__, 'page/seo_setting.tpl') : '',
            ]);
        }
    }

    public function seo_supplier_html()
    {
        $this->assignPageParams('supplier');
        $enableSeo = 1;
        if ($this->getRequestContainer()) {
            $controller = Tools::getValue('controller');
            if ('AdminSuppliers' == $controller) {
                $this->addTwigVar('ets_supplier_seo_setting_html', $enableSeo ? $this->display(__FILE__, 'page/seo_setting.tpl') : '');
            }
        } else {
            $this->context->smarty->assign([
                'ets_supplier_seo_setting_html' => $enableSeo ? $this->display(__FILE__, 'page/seo_setting.tpl') : '',
            ]);
        }
    }

    /**
     * seo_cms_html.
     *
     * @return void
     */
    public function seo_cms_html()
    {
        $this->assignPageParams('cms');
        $enableSeo = 1;
        if ($this->getRequestContainer()) {
            $controller = Tools::getValue('controller');
            if ('AdminCmsContent' == $controller) {
                $this->addTwigVar('ets_cms_seo_setting_html', $enableSeo ? $this->display(__FILE__, 'page/seo_setting.tpl') : '');
            }
        } else {
            $this->context->smarty->assign([
                'ets_cms_seo_setting_html' => $enableSeo ? $this->display(__FILE__, 'page/seo_setting.tpl') : '',
            ]);
        }
    }

    public function seo_cms_category_html()
    {
        $this->assignPageParams('cms_category');
        $enableSeo = 1;
        if ($this->getRequestContainer()) {
            $controller = Tools::getValue('controller');
            if ('AdminCmsContent' == $controller) {
                $this->addTwigVar('ets_cms_category_seo_setting_html', $enableSeo ? $this->display(__FILE__, 'page/seo_setting.tpl') : '');
            }
        } else {
            $this->context->smarty->assign([
                'ets_cms_category_seo_setting_html' => $enableSeo ? $this->display(__FILE__, 'page/seo_setting.tpl') : '',
            ]);
        }
    }

    /**
     * seo_cms_html.
     *
     * @return void
     */
    public function seo_meta_html()
    {
        $this->assignPageParams('meta');
        $enableSeo = 1;
        if ($this->getRequestContainer()) {
            $controller = Tools::getValue('controller');
            if ('AdminMeta' == $controller) {
                $this->addTwigVar('ets_meta_seo_setting_html', $enableSeo ? $this->display(__FILE__, 'page/seo_setting.tpl') : '');
                $this->addTwigVar('ETS_AWU_ENABLE_REMOVE_ID_IN_URL', (int) Configuration::get('ETS_AWU_ENABLE_REMOVE_ID_IN_URL'));
                $this->addTwigVar('ETS_AWU_ENABLE_REMOVE_ATTR_ALIAS', (int) Configuration::get('ETS_AWU_ENABLE_REMOVE_ATTR_ALIAS'));
                $this->addTwigVar('ETS_AWU_ENABLE_REMOVE_ID_ATTR_ALIAS', (int) Configuration::get('ETS_AWU_ENABLE_REMOVE_ID_ATTR_ALIAS'));
                $this->addTwigVar('ETS_AWU_ENABLE_REDRECT_NOTFOUND', (int) Configuration::get('ETS_AWU_ENABLE_REDRECT_NOTFOUND'));
                $this->addTwigVar('ETS_AWU_REDIRECT_STATUS_CODE', (int) Configuration::get('ETS_AWU_REDIRECT_STATUS_CODE'));
                $this->addTwigVar('ETS_AWU_ENABLE_REMOVE_LANG_CODE_IN_URL', (int) Configuration::get('ETS_AWU_ENABLE_REMOVE_LANG_CODE_IN_URL'));
                $this->addTwigVar('titleRemoveLangCode', $this->l('Remove ISO code in URL for default language'));
                $this->addTwigVar('titleRemoveAttrAlias', $this->l('Remove attribute alias in URL'));
            }
        } else {
            $this->context->smarty->assign([
                'ets_meta_seo_setting_html' => $enableSeo ? $this->display(__FILE__, 'page/seo_setting.tpl') : '',
            ]);
        }
    }

    public function getSeoMetaData()
    {
        $page = $this->is17 ? $this->context->controller->getTemplateVarPage() : [];
        if (($controller = Tools::getValue('controller', null)) && Validate::isControllerName($controller)) {
            $id_lang = $this->context->language->id;
            $dataSeo = [];
            if (($id_product = (int) Tools::getValue('id_product')) && 'product' == $controller) {
                $dataSeo = EtsAwuProduct::getSeoProduct($id_product, $this->context, $id_lang);
            } elseif (($id_category = (int) Tools::getValue('id_category')) && 'category' == $controller) {
                $dataSeo = EtsAwuCategory::getSeoCategory($id_category, $this->context, $id_lang);
            } elseif (($id_cms = (int) Tools::getValue('id_cms')) && 'cms' == $controller) {
                $dataSeo = EtsAwuCms::getSeoCms($id_cms, $this->context, $id_lang);
            } elseif (($id_cms = (int) Tools::getValue('id_cms_category')) && in_array($controller, ['cms', 'cms_category'])) {
                $dataSeo = EtsAwuCmsCategory::getSeoCmsCategory($id_cms, $this->context, $id_lang);
            } elseif (($id_manufaturer = (int) Tools::getValue('id_manufacturer')) && 'manufacturer' == $controller) {
                $dataSeo = EtsAwuManufacturer::getSeoManufacturer($id_manufaturer, $this->context, $id_lang);
            } elseif (($id_supplier = (int) Tools::getValue('id_supplier')) && 'supplier' == $controller) {
                $dataSeo = EtsAwuSupplier::getSeoSupplier($id_supplier, $this->context, $id_lang);
            } else {
                $meta = Meta::getMetaByPage($this->context->controller->php_self, $id_lang);
                if ($meta) {
                    $dataSeo = EtsAwuMeta::getSeoMeta((int) $meta['id_meta'], $this->context, $id_lang);
                }
            }
            if (!$dataSeo) {
                return '';
            }

            $meta_robot_default = isset($page['meta']['robots']) && $page['meta']['robots'] ? explode(',', $page['meta']['robots']) : [];
            $allow_search = $dataSeo ? (int) $dataSeo['allow_search'] : 1;
            $allow_flw_link = $dataSeo ? (int) $dataSeo['allow_flw_link'] : 1;
            $canonical_url = $dataSeo && $dataSeo['canonical_url'] ? $dataSeo['canonical_url'] : '';

            $meta_robot = $dataSeo ? $dataSeo['meta_robots_adv'] : '';
            $meta_robot = explode(',', $meta_robot);
            if (in_array('default', $meta_robot)) {
                if ('product' == $controller) {
                    $meta_robot_default[] = 'index';
                }
            } elseif (in_array('none', $meta_robot)) {
                $meta_robot_default = [];
            } else {
                $meta_robot_default = [];
                if (in_array('noarchive', $meta_robot)) {
                    $meta_robot_default[] = 'noarchive';
                }
                if (in_array('nosnippet', $meta_robot)) {
                    $meta_robot_default[] = 'nosnippet';
                }
                if (in_array('noimageindex', $meta_robot)) {
                    $meta_robot_default[] = 'noimageindex';
                }
            }

            if (!$allow_search) {
                $meta_robot_default[] = 'noindex';
                foreach ($meta_robot_default as $k => $rb) {
                    if ('index' == $rb) {
                        unset($meta_robot_default[$k]);
                    }
                }
                if (!$this->is17) {
                    $this->context->smarty->assign('nobots', true);
                }
            } elseif (!$this->is17) {
                $this->context->smarty->clearAssign('nobots');
            }
            if (!$allow_flw_link) {
                $meta_robot_default[] = 'nofollow';
                foreach ($meta_robot_default as $k => $rb) {
                    if ('follow' == $rb) {
                        unset($meta_robot_default[$k]);
                    }
                }
                if (!$this->is17) {
                    $this->context->smarty->assign('nofollow', true);
                }
            } elseif (!$this->is17) {
                $this->context->smarty->clearAssign('nofollow');
            }
            $page['meta']['robots'] = implode(',', $meta_robot_default);
            if (trim($canonical_url)) {
                $page['canonical'] = $canonical_url;
                if (!$this->is17) {
                    $this->context->smarty->assign('canonical', $canonical_url);
                }
            }
        }

        if ($this->is17) {
            $this->context->smarty->assign([
                'page' => $page,
            ]);
        }
    }

    public static function validateArray($array, $validate = 'isCleanHtml')
    {
        if (!is_array($array)) {
            return true;
        }
        if (method_exists('Validate', $validate)) {
            if ($array && is_array($array)) {
                $ok = true;
                foreach ($array as $val) {
                    if (!is_array($val)) {
                        if ($val && !Validate::$validate($val)) {
                            $ok = false;
                            break;
                        }
                    } else {
                        $ok = self::validateArray($val, $validate);
                    }
                }

                return $ok;
            }
        }

        return true;
    }

    public function validateLinkRewrite($params)
    {
        if (isset($params['object'])) {
            $type = null;
            $obj = $params['object'];
            $idCol = '';
            if ($obj instanceof Product) {
                $type = 'product';
                $idCol = 'id_product';
            } elseif ($obj instanceof Category) {
                $type = 'category';
                $idCol = 'id_category';
            } elseif ($obj instanceof CMS) {
                $type = 'cms';
                $idCol = 'id_cms';
            } elseif ($obj instanceof CMSCategory) {
                $type = 'cms_category';
                $idCol = 'id_cms_category';
            } elseif ($obj instanceof Meta) {
                $type = 'meta';
                $idCol = 'id_meta';
            }
            if (!$type) {
                return;
            }
            $error = null;
            $seoAdvanced = Tools::getValue('ets_awu_advanced');
            if (isset($seoAdvanced['canonical_url']) && !$error) {
                foreach ($seoAdvanced['canonical_url'] as $id_lang => $url) {
                    if ($url && !Validate::isAbsoluteUrl($url)) {
                        $error = sprintf($this->l('[%s] The Canonical url must start with http:// or https:// '), Language::getIsoById($id_lang));
                        break;
                    }
                }
            }
            if ($error) {
                if ($this->is17) {
                    if ('cms' !== $type) {
                        throw new PrestaShopException($error);
                    } else {
                        $controller = ($controller = Tools::getValue('controller')) && Validate::isCleanHtml($controller) ? $controller : '';
                        $this->context->cookie->__set('ets_awu_error_link_rewrite', $error);
                        if ($obj->id) {
                            $redirectUrl = $this->context->link->getAdminLink($controller, true, [], [
                                $idCol => $obj->id,
                                'updatecms' => true,
                            ]) . '&' . $idCol . '=' . $obj->id . '&update' . $type;
                        } else {
                            $redirectUrl = $this->context->link->getAdminLink($controller, true, [], [
                                $idCol => $obj->id,
                                'id_cms_category' => $obj->id_cms_category,
                                'addcms' => true,
                            ]);
                        }
                        Tools::redirectAdmin($redirectUrl);
                    }
                } else {
                    $controller = ($controller = Tools::getValue('controller')) && Validate::isCleanHtml($controller) ? $controller : '';
                    $this->context->cookie->__set('ets_awu_error_link_rewrite', $error);
                    if ($obj->id) {
                        $redirectUrl = $this->context->link->getAdminLink($controller, true) . '&' . $idCol . '=' . $obj->id . '&update' . $type;
                    } else {
                        $redirectUrl = $this->context->link->getAdminLink($controller, true) . '&add' . $type;
                    }
                    Tools::redirectAdmin($redirectUrl);
                }
            }
        }
    }

    public function formHandleLinkRewrite($params, $type)
    {
        $id = isset($params['id']) ? (int) $params['id'] : null;
        $redirectUrl = null;
        switch ($type) {
            case 'category':
                if ($id) {
                    $redirectUrl = $this->context->link->getAdminLink(
                        'AdminCategories',
                        true,
                        ['route' => 'admin_categories_edit', 'categoryId' => $id],
                        ['id_category' => $id, 'updatecategory' => true]
                    );
                } else {
                    $idParent = (int) $params['form_data']['id_parent'];
                    $redirectUrl = $this->context->link->getAdminLink(
                        'AdminCategories',
                        true,
                        ['route' => 'admin_categories_create', 'id_parent' => $idParent],
                        ['addcategory' => true]
                    );
                }
                break;
            case 'cms':
                if ($id) {
                    $redirectUrl = $this->context->link->getAdminLink(
                        'AdminCms',
                        true,
                        ['route' => 'admin_cms_pages_edit', 'cmsPageId' => $id],
                        ['id_cms' => $id, 'updatecms' => true]
                    );
                } else {
                    $idParent = (int) (isset($params['form_data']['page_category_id']) && $params['form_data']['page_category_id'] ? $params['form_data']['page_category_id'] : $params['form_data']['id_cms_category']);
                    $redirectUrl = $this->context->link->getAdminLink(
                        'AdminCms',
                        true,
                        ['route' => 'admin_cms_pages_create', 'id_cms_category' => $idParent],
                        ['addcms' => true, 'id_cms_category' => $idParent]
                    );
                }
                break;
            case 'cms_category':
                if ($id) {
                    $redirectUrl = $this->context->link->getAdminLink(
                        'AdminCmsCategories',
                        true,
                        ['route' => 'admin_cms_pages_category_edit', 'cmsCategoryId' => $id],
                        ['id_cms_category' => $id, 'updatecms_category' => true]
                    );
                } else {
                    $redirectUrl = $this->context->link->getAdminLink(
                        'AdminCmsCategories',
                        true,
                        ['route' => 'admin_cms_pages_category_create'],
                        ['addcms_category' => true]
                    );
                }
                break;
            case 'meta':
                if ($id) {
                    $redirectUrl = $this->context->link->getAdminLink(
                        'AdminMeta',
                        true,
                        ['route' => 'admin_metas_edit', 'metaId' => $id],
                        ['id_meta' => $id, 'updatemeta' => true]
                    );
                } else {
                    $redirectUrl = $this->context->link->getAdminLink(
                        'AdminMeta',
                        true,
                        ['route' => 'admin_metas_create'],
                        ['addmeta' => true]
                    );
                }
                break;
        }
        $error = null;
        $seoAdvanced = Tools::getValue('ets_awu_advanced');
        if (isset($seoAdvanced['canonical_url']) && !$error) {
            foreach ($seoAdvanced['canonical_url'] as $id_lang => $url) {
                if ($url && !Validate::isAbsoluteUrl($url)) {
                    $error = sprintf($this->l('[%s] The Canonical url must start with http:// or https:// '), Language::getIsoById($id_lang));
                    break;
                }
            }
        }
        if ($error) {
            $params['form_data']['ets_awu_error'] = $error;
            $fileName = time() . rand(1111, 99999) . '.json';
            file_put_contents(dirname(__FILE__) . '/cache/' . $fileName, json_encode($params['form_data']));
            $this->context->cookie->ets_awu_form_validate_data = $fileName;
            $this->context->cookie->write();
            Tools::redirectAdmin($redirectUrl);
        }
    }

    public function setFormBuilderModifier(&$params)
    {
        if ($fileData = $this->context->cookie->__get('ets_awu_form_validate_data')) {
            $data = [];
            if (file_exists(dirname(__FILE__) . '/cache/' . $fileData)) {
                $json = Tools::file_get_contents(dirname(__FILE__) . '/cache/' . $fileData);
                $data = json_decode($json, true);
                unlink(dirname(__FILE__) . '/cache/' . $fileData);
            }
            $error = '';
            if (isset($data['ets_awu_error'])) {
                $error = $data['ets_awu_error'];
                unset($data['ets_awu_error']);
            }
            $params['data'] = array_merge($params['data'], $data);
            $params['form_builder']->setData($params['data']);
            $this->context->cookie->__unset('ets_awu_form_validate_data');
            $this->context->cookie->__set('ets_awu_error_link_rewrite', $error);
        }
    }

    public function hookActionObjectAddBefore($params)
    {
        if (!defined('_PS_ADMIN_DIR_')) {
            return;
        }
        if ($this->getRequestContainer()) {
            // Removed
        } else {
            $this->validateLinkRewrite($params);
        }
    }

    /* == Category === */
    public function hookActionBeforeUpdateCategoryFormHandler($params)
    {
        $this->formHandleLinkRewrite($params, 'category');
    }

    public function hookActionBeforeCreateCategoryFormHandler($params)
    {
        $this->formHandleLinkRewrite($params, 'category');
    }

    public function hookActionCategoryFormBuilderModifier($params)
    {
        $this->setFormBuilderModifier($params);
    }

    // Root category
    public function hookActionBeforeUpdateRootCategoryFormHandler($params)
    {
        $this->formHandleLinkRewrite($params, 'category');
    }

    public function hookActionBeforeCreateRootCategoryFormHandler($params)
    {
        $this->formHandleLinkRewrite($params, 'category');
    }

    public function hookActionRootCategoryFormBuilderModifier($params)
    {
        $this->setFormBuilderModifier($params);
    }

    /* = CMS == */
    public function hookActionBeforeUpdateCmsPageFormHandler($params)
    {
        $this->formHandleLinkRewrite($params, 'cms');
    }

    public function hookActionBeforeCreateCmsPageFormHandler($params)
    {
        $this->formHandleLinkRewrite($params, 'cms');
    }

    public function hookActionCmsPageFormBuilderModifier($params)
    {
        $this->setFormBuilderModifier($params);
    }

    /* = CMS Category == */
    public function hookActionBeforeUpdateCmsPageCategoryFormHandler($params)
    {
        $this->formHandleLinkRewrite($params, 'cms_category');
    }

    public function hookActionBeforeCreateCmsPageCategoryFormHandler($params)
    {
        $this->formHandleLinkRewrite($params, 'cms_category');
    }

    public function hookActionCmsPageCategoryFormBuilderModifier($params)
    {
        $this->setFormBuilderModifier($params);
    }

    /* = Meta == */
    public function hookActionBeforeUpdateMetaFormHandler($params)
    {
        $this->formHandleLinkRewrite($params, 'meta');
    }

    public function hookActionBeforeCreateMetaFormHandler($params)
    {
        $this->formHandleLinkRewrite($params, 'meta');
    }

    public function hookActionMetaFormBuilderModifier($params)
    {
        $this->setFormBuilderModifier($params);
    }

    public function hookActionProductFormBuilderModifier($params)
    {
        if (isset($params['form_builder'], $params['id']) && $params['form_builder'] instanceof \Symfony\Component\Form\FormBuilder && $params['id']) {
            require_once __DIR__ . '/src/FormType/SeoType.php';
            if (@file_exists(__DIR__ . '/../ets_marketplace/src/FormType/DescType.php')) {
                require_once __DIR__ . '/../ets_marketplace/src/FormType/DescType.php';
            }
        }
    }
}
